package ANPClient;
#Modified by Ana Sanz Merino


BEGIN
{
    my $CLIENT_MODULE_INCLUDE_PATH = './';

    @INC = (@INC, $CLIENT_MODULE_INCLUDE_PATH);
}

use SOAP::Lite;
use XML::LibXML;
use HTTP::Cookies;
use Time::HiRes qw(gettimeofday);
use PolicyEngine;

my $MEASURING_COMPONENTS = 1;
# MEASURING_CASES is now exclusive to the login() method and inferred from that methods arguments.
my $MEASURING_POLICY_ENGINE = 1;

my $SEND_NONCE = 0;

# Use 2 ** 30 just to be safe
my $MAX_NONCE_VALUE = 2 ** 30;



my $PRINT_ERROR = 0;
my $PRINT_WARNINGS = 0;
my $PRINT_INFORMATION = 0;
my $PRINT_MSGS = 0;
my $PRINT_MEASUREMENT = 0;

my $PRINT_PROGRAM_NAME = 'ANPClient';
my $PRINT_ERROR_PREFIX = 'ERROR - ' . $PRINT_PROGRAM_NAME . ' - ';
my $PRINT_WARNING_PREFIX = 'WARNING - ' . $PRINT_PROGRAM_NAME . ' - ';
my $PRINT_INFORMATION_PREFIX = 'INFORMATION - ' . $PRINT_PROGRAM_NAME . ' - ';
my $PRINT_MEASUREMENT_PREFIX = 'MEASUREMENT - ' . $PRINT_PROGRAM_NAME . ' - ';



my $SOAP_PROXY = 'http://wizard.cs.berkeley.edu:8080/Sahara_SSO_sp/soap';



my $ANP_NAMESPACE = 'urn:ucb:eecs:sahara:sso:ANP:1.0';
my $SAML_NAMESPACE = 'urn:oasis:names:tc:SAML:1.0:assertion';
my $SAMLP_NAMESPACE = 'urn:oasis:names:tc:SAML:1.0:protocol';
my $ANP_PREFIX = 'anp';
my $SAML_PREFIX = 'saml';



my $AUTH_CAPABILITIES_QUERY_ELEMENT_NAME = 'AuthnCapabilitiesQuery';
my $AUTH_QUERY_ELEMENT_NAME = 'AuthnQuery';
my $AUTH_CONT_QUERY_ELEMENT_NAME = 'AuthnContQuery';
my $AUTH_ID_REFERENCE_ELEMENT_NAME = 'AuthnMethodIDReference';
my $AUTH_METHOD_ELEMENT_NAME = 'AuthnMethod';
my $AUTH_METHOD_ID_ELEMENT_NAME = 'AuthnMethodID';
my $AUTH_RESULT_ELEMENT_NAME = 'AuthnResult';
my $AUTH_STATUS_CODE_ELEMENT_NAME = 'StatusCode';
my $AUTH_STATUS_CODE_VALUE_ATTRIBUTE_NAME = 'Value';

my $CHARGING_ID_REFERENCE_ELEMENT_NAME = 'ChargingOptionIDReference';
my $CHARGING_OPTION_ELEMENT_NAME = 'ChargingOption';
my $CHARGING_OPTION_ID_ELEMENT_NAME = 'ChargingOptionID';

my $USER_INFO_DESIGNATOR_ELEMENT_NAME = 'UserInfoDesignator';
my $USER_INFO_ATTRIBUTE_NAME_ATTRIBUTE_NAME = 'AttributeName';
my $USER_INFO_ATTRIBUTE_VALUE_ATTRIBUTE_NAME = 'AttributeValue';
my $USER_INFO_AUTH_QUERY_KEY_ATTRIBUTE_NAME = 'Key';
my $USER_INFO_MAC_ADDRESS_ATTRIBUTE_NAME = 'MacAddr';
my $USER_INFO_ELEMENT_NAME = 'UserInfo';

my $REDIRECT_URL_ELEMENT_NAME = 'RedirectURL';
my $REDIRECTED_URL_ELEMENT_NAME = 'RedirectedURL';

my $NONCE_ELEMENT_NAME = 'Nonce';



my $SUCCESS_AUTH_RESULT = 'Successful';
my $FAILURE_AUTH_RESULT = 'Failed';
my $REDIRECT_AUTH_RESULT = 'Redirect';

my $SUCCESSFUL_STATUS_CODE = 'samlp:Success';

my $NORMAL_RETURN_STATUS = 1;
my $ERROR_RETURN_STATUS = 0;



my $REQUEST_METHOD_NAME = 'Request';
my $REQUEST_METHOD_MAJOR_VERSION = 1;
my $REQUEST_METHOD_MINOR_VERSION = 0;



my $HEADER_NAME_COMPONENT_INDEX = 0;
my $HEADER_VALUE_COMPONENT_INDEX = 1;



my $MEASUREMENT_DESCRIPTIONS_KEY = 'measurementDescriptions';
my $MEASUREMENT_TIMES_KEY = 'measurementTimes';

my $POLICY_ENGINE_KEY = 'policyEngine';



my $CUSTOM_ID_PROVIDER_DOMAIN_KEY = 'IDProviderDomain';
my $CUSTOM_AUTH_METHOD_KEY = 'authMethodID';
my $CUSTOM_CHARGING_OPTION_KEY = 'chargingOptionID';



my $TRUE = 1;
my $FALSE = 0;



my $LOGIN_INFO_USER_INFO_TYPE = 'loginInfo';

my $USER_NAME_USER_INFO_NAME = 'userName';
my $PASSWORD_USER_INFO_NAME = 'password';


# This is for temporary storage of the return status of a method call.
my $returnStatus;



# This method creates a new ANPClient instance.
sub new
{
    my $PRINT_LOCAL_PREFIX = 'new() - ';

    my $class = shift;

    my $policyEngine = PolicyEngine->new();
    my $this = { $POLICY_ENGINE_KEY => $policyEngine };

    bless($this, $class);

    return $this;
}



sub login
{
    my $PRINT_LOCAL_PREFIX = 'login() - ';


    my $this = shift;

    #MEASUREMENT CODE
    my $measurementDescriptions = [ ];
    my $measurementTimes = [ ];
    if ($MEASURING_COMPONENTS)
    {
	$this->{$MEASUREMENT_DESCRIPTIONS_KEY} = $measurementDescriptions;
	$this->{$MEASUREMENT_TIMES_KEY} = $measurementTimes;

	push(@$measurementDescriptions, "10. Starting login");
	push(@$measurementTimes, gettimeofday());
    }

    my $authQueryKey = shift;
    my $MACAddress = shift;
    
    my %customValues =  @_;

    if ($PRINT_INFORMATION)
    {
	print(STDERR $PRINT_INFORMATION_PREFIX . $PRINT_LOCAL_PREFIX . "Key = " . $authQueryKey .
	      ", MAC Address = " . $MACAddress . "\n");
    }

    # Send AuthCapabilitiesQuery
    my $capabilitiesQuerySOAP = SOAP::Lite
	->outputxml($TRUE)
	->uri($ANP_NAMESPACE)
	->proxy($SOAP_PROXY);
	#TODO - SSL
    my $requestSOAPMethod = SOAP::Data->name($REQUEST_METHOD_NAME)
	->attr({MajorVersion => $REQUEST_METHOD_MAJOR_VERSION , MinorVersion => $REQUEST_METHOD_MINOR_VERSION})
	->uri($ANP_NAMESPACE)
	->prefix($ANP_PREFIX);

    my $capabilitiesQuery;
    if ($SEND_NONCE) {
        my $capabilitiesQueryNonce = int(rand($MAX_NONCE_VALUE));
        $capabilitiesQuery = SOAP::Data->name($AUTH_CAPABILITIES_QUERY_ELEMENT_NAME =>
	   				         \SOAP::Data->name($NONCE_ELEMENT_NAME =>
							           $capabilitiesQueryNonce)
					         ->uri($ANP_NAMESPACE)
					         ->prefix($ANP_PREFIX))
        ->prefix($ANP_PREFIX);
    } else {
        $capabilitiesQuery = SOAP::Data->name($AUTH_CAPABILITIES_QUERY_ELEMENT_NAME)
            ->uri($ANP_NAMESPACE)
            ->prefix($ANP_PREFIX);
    }

    if ($PRINT_INFORMATION)
    {
	print(STDERR $PRINT_INFORMATION_PREFIX . $PRINT_LOCAL_PREFIX .
	      "Sending Auth Capabilities Query with nonce = \'" . $capabilitiesQueryNonce . "\'.\n");
    }

    if ($MEASURING_COMPONENTS)
    {
	push(@$measurementDescriptions, "20. About to send capabilities statement request");
	push(@$measurementTimes, gettimeofday());
    }

    my $capabilitiesStatement = $capabilitiesQuerySOAP
	->call($requestSOAPMethod => $capabilitiesQuery);

    if ($MEASURING_COMPONENTS)
    {
	push(@$measurementDescriptions, "30. Done sending capabilities statement");
	push(@$measurementTimes, gettimeofday());
    }

    if ($PRINT_MSGS)
    {
	print (STDERR $capabilitiesStatement . "\n");
    }

    # Policy engine interconnect
    my $policyEngine = $this->{$POLICY_ENGINE_KEY};

    if ($MEASURING_COMPONENTS)
    {
	push(@$measurementDescriptions, "40. About to execute policy engine");
	push(@$measurementTimes, gettimeofday());
    }
    
    my $policyEngineReturnStatus = $TRUE;
    my $selectedIDProviderDomainString = "NULL";
    my $selectedAuthMethodIDString = "NULL";
    my $selectedChargingOptionIDString = "NULL";
    my $requiredInfoDesignatorNames = [ ];
    my $requiredInfoDesignatorValues = [ ];

    if ($MEASURING_POLICY_ENGINE)
    {
	($policyEngineReturnStatus, $selectedIDProviderDomainString, $selectedAuthMethodIDString,
	 $selectedChargingOptionIDString, $requiredInfoDesignatorNames, $requiredInfoDesignatorValues) =
	     $policyEngine->choose($capabilitiesStatement, $capabilitiesQueryNonce);
    }
	
    if ($MEASURING_COMPONENTS)
    {
	push(@$measurementDescriptions, "50. Done executing policy engine");
	push(@$measurementTimes, gettimeofday());
    }
    
    foreach my $currentCustomKey (keys(%customValues))
    {
	if ($PRINT_INFORMATION)
	{
	    print(STDERR $PRINT_INFORMATION_PREFIX . $PRINT_LOCAL_PREFIX .
		  "Looking at custom key = \'" . $currentCustomKey . "\'.\n");
	}

	if ($currentCustomKey eq $CUSTOM_ID_PROVIDER_DOMAIN_KEY)
	{
	    $selectedIDProviderDomainString = $customValues{$CUSTOM_ID_PROVIDER_DOMAIN_KEY};

	    if ($PRINT_INFORMATION)
	    {
		print(STDERR $PRINT_INFORMATION_PREFIX . $PRINT_LOCAL_PREFIX .
		      "Custom set ID Provider to \'" . $selectedIDProviderDomainString . "\'.\n");
	    }
	}
	elsif ($currentCustomKey eq $CUSTOM_AUTH_METHOD_KEY)
	{
	    $selectedAuthMethodIDString = $customValues{$CUSTOM_AUTH_METHOD_KEY};

	    if ($PRINT_INFORMATION)
	    {
		print(STDERR $PRINT_INFORMATION_PREFIX . $PRINT_LOCAL_PREFIX .
		      "Custom set auth method to \'" . $selectedAuthMethodIDString . "\'.\n");
	    }
	}
	elsif ($currentCustomKey eq $CUSTOM_CHARGING_OPTION_KEY)
	{
	    $selectedChargingOptionIDString = $customValues{$CUSTOM_CHARGING_OPTION_KEY};

	    if ($PRINT_INFORMATION)
	    {
		print(STDERR $PRINT_INFORMATION_PREFIX . $PRINT_LOCAL_PREFIX .
		      "Custom set charging option to \'" . $selectedChargingOptionIDString . "\'.\n");
	    }
	}
	else
	{
	    my $currentInfoDesignatorIndex;
	    for ($currentInfoDesignatorIndex = 0;
		 $currentInfoDesignatorIndex < scalar(@$requiredInfoDesignatorNames);
		 $currentInfoDesignatorIndex ++)
	    {
		if ($currentCustomKey eq $requiredInfoDesignatorNames->[$currentInfoDesignatorIndex])
		{
		    $requiredInfoDesignatorValues->[$currentInfoDesignatorIndex] =
			$customValues{$currentCustomKey};

		    if ($PRINT_INFORMATION)
		    {
			print(STDERR $PRINT_INFORMATION_PREFIX . $PRINT_LOCAL_PREFIX .
			      "Custom set info designator \'" . $currentCustomKey . "\' to \'" .
			      $requiredInfoDesignatorValues->[$currentInfoDesignatorIndex] . "\'.\n");
		    }
		    
		    last;
		}
	    }
	    if ($currentInfoDesignatorIndex == scalar(@$requiredInfoDesignatorNames))
	    {
		push(@$requiredInfoDesignatorNames, $currentCustomKey);
		push(@$requiredInfoDesignatorValues, $customValues{$currentCustomKey});

		if ($PRINT_INFORMATION)
		{
		    print(STDERR $PRINT_INFORMATION_PREFIX . $PRINT_LOCAL_PREFIX .
			  "Custom added info designator \'" .
			  $requiredInfoDesignatorNames->[scalar(@$requiredInfoDesignatorNames) - 1] .
			  "\' with value \'" .
			  $requiredInfoDesignatorValues->[scalar(@$requiredInfoDesignatorValues) - 1] .
			  "\'.\n");
		}
	    }
	}
    }

    if (!($policyEngineReturnStatus))
    {
	if ($PRINT_ERROR)
	{
	    print(STDERR $PRINT_ERROR_PREFIX . $PRINT_LOCAL_PREFIX .
		  "Policy engine failed; Returning failure result.\n");
	}
	
	if ($MEASURING_COMPONENTS)
	{
	    push(@$measurementDescriptions, "55. Exiting");
	    push(@$measurementTimes, gettimeofday());
	}
	
	return $FAILURE_AUTH_RESULT;
    }
    
    
    if ($PRINT_INFORMATION)
    {
	print(STDERR $PRINT_INFORMATION_PREFIX . $PRINT_LOCAL_PREFIX . "ID Provider domain = " .
	      $selectedIDProviderDomainString . "\n");
	print(STDERR $PRINT_INFORMATION_PREFIX . $PRINT_LOCAL_PREFIX . "Authentication method ID = " .
	      $selectedAuthMethodIDString . "\n");
	print(STDERR $PRINT_INFORMATION_PREFIX . $PRINT_LOCAL_PREFIX . "Charging option ID = " .
	      $selectedChargingOptionIDString . "\n");

	for (my $currentRequiredInfoDesignatorIndex = 0;
	     $currentRequiredInfoDesignatorIndex < scalar(@$requiredInfoDesignatorNames);
	     $currentRequiredInfoDesignatorIndex ++)
	{
	    print(STDERR $PRINT_INFORMATION_PREFIX . $PRINT_LOCAL_PREFIX .
		  "Info designator name = \'" .
		  $requiredInfoDesignatorNames->[$currentRequiredInfoDesignatorIndex] .
		  "\', value = \'" .
		  $requiredInfoDesignatorValues->[$currentRequiredInfoDesignatorIndex] .
		  "\'.\n");
	}
    }

    if ($MEASURING_COMPONENTS)
    {
	push(@$measurementDescriptions, "60. Starting building of auth query");
	push(@$measurementTimes, gettimeofday());
    }

    # Begin building the AuthQuery
    # The querySOAPWithCookies will store the cookies from the AuthnStatement (The response to the AuthnQuery; The query
    #   we are making here) and, if appropriate, will be used again to make the AuthnContQuery (Part of Liberty auth),
    #   into which it will insert these cookies
    my $queryCookies = HTTP::Cookies->new();

    my $querySOAPWithCookies = SOAP::Lite
	->autotype(0)
	->outputxml(1)
	->uri($ANP_NAMESPACE)
	->proxy($SOAP_PROXY,
		cookie_jar => $queryCookies);

    my @authQueryInfo;

    # Send the auth method ID, the charging option ID, the auth query key, and the MAC address.
    push(@authQueryInfo,
	 SOAP::Data->name($AUTH_ID_REFERENCE_ELEMENT_NAME => $selectedAuthMethodIDString)
	 ->prefix($ANP_PREFIX));
    push(@authQueryInfo,
	 SOAP::Data->name($CHARGING_ID_REFERENCE_ELEMENT_NAME => $selectedChargingOptionIDString)
	 ->prefix($ANP_PREFIX));
    push(@authQueryInfo,
	 SOAP::Data->name($USER_INFO_ELEMENT_NAME =>
			  \SOAP::Data->name($USER_INFO_ATTRIBUTE_VALUE_ATTRIBUTE_NAME => $authQueryKey)
			  ->prefix($SAML_PREFIX)
			  ->uri($SAML_NAMESPACE))
	 ->prefix($ANP_PREFIX)
	 ->attr({AttributeName => $USER_INFO_AUTH_QUERY_KEY_ATTRIBUTE_NAME}));
    push(@authQueryInfo,
	 SOAP::Data->name($USER_INFO_ELEMENT_NAME =>
			  \SOAP::Data->name($USER_INFO_ATTRIBUTE_VALUE_ATTRIBUTE_NAME => $MACAddress)
			  ->prefix($SAML_PREFIX)
			  ->uri($SAML_NAMESPACE))
	 ->prefix($ANP_PREFIX)
	 ->attr({AttributeName => $USER_INFO_MAC_ADDRESS_ATTRIBUTE_NAME}));
    
    for (my $currentInfoDesignatorIndex = 0;
	 $currentInfoDesignatorIndex < scalar(@$requiredInfoDesignatorNames);
	 $currentInfoDesignatorIndex ++)
    {
	$currentInfoDesignatorName = $requiredInfoDesignatorNames->[$currentInfoDesignatorIndex];
	$associatedInfoDesignatorValue = $requiredInfoDesignatorValues->[$currentInfoDesignatorIndex];
	
	push(@authQueryInfo,
	     SOAP::Data->name($USER_INFO_ELEMENT_NAME =>
			      \SOAP::Data->name($USER_INFO_ATTRIBUTE_VALUE_ATTRIBUTE_NAME =>
						$associatedInfoDesignatorValue)
			      ->prefix($SAML_PREFIX)
			      ->uri($SAML_NAMESPACE))
	     ->prefix($ANP_PREFIX)
	     ->attr({AttributeName => $currentInfoDesignatorName}));
    }

    # Finish building the AuthQuery and send it
    my $authQuery =  SOAP::Data->name($AUTH_QUERY_ELEMENT_NAME => 
				      \SOAP::Data->value(@authQueryInfo)
				      ->prefix($ANP_PREFIX));

    if ($MEASURING_COMPONENTS)
    {
	push(@$measurementDescriptions, "70. About to send auth query");
	push(@$measurementTimes, gettimeofday());
    }

    my $authStatement = $querySOAPWithCookies->call($requestSOAPMethod => $authQuery);

    if ($MEASURING_COMPONENTS)
    {
	push(@$measurementDescriptions, "80. Done sending auth query");
	push(@$measurementTimes, gettimeofday());
    }

    if ($PRINT_MSGS)
    {
	print (STDERR $authStatement . "\n");
    }

    if ($PRINT_INFORMATION)
    {
	print(STDERR $PRINT_INFORMATION_PREFIX . $PRINT_LOCAL_PREFIX . "Cookies extracted from the AuthnStatement:\n" .
	      $queryCookies->as_string() . "\n");
    }



    # This call will return only if further analysis is required to determine the auth outcome
    my $authOutcome = $this->determineAuthOutcome($authStatement);
    if (($authOutcome eq $SUCCESS_AUTH_RESULT) || ($authOutcome eq $FAILURE_AUTH_RESULT))
    {
	if ($MEASURING_COMPONENTS)
	{
	    push(@$measurementDescriptions, "90. Exiting");
	    push(@$measurementTimes, gettimeofday());
	}

	return $authOutcome;
    }

    if ($PRINT_INFORMATION)
    {
	print(STDERR $PRINT_INFORMATION_PREFIX . $PRINT_LOCAL_PREFIX .
	      "Outcome of initial auth was ambigous; Continuing to Liberty\n");
    }



    my $authStatementParser = XML::LibXML->new();
    
    my $authStatementParseTree = $authStatementParser->parse_string($authStatement);

    my ($authResult) = $authStatementParseTree->getElementsByTagNameNS($ANP_NAMESPACE, $AUTH_RESULT_ELEMENT_NAME);



    # The only chance left is Liberty authentication - There must be an AuthnResult element
    #   with a value of "Redirect"
    if ($authResult && ($authResult->textContent eq $REDIRECT_AUTH_RESULT))
    {
	if ($MEASURING_COMPONENTS)
	{
	    push(@$measurementDescriptions, "100. Starting Liberty auth");
	    push(@$measurementTimes, gettimeofday());
	}

	# Get the URL specified by the RedirectURL element
	my ($IDProviderRedirectURL) = $authStatementParseTree
	    ->getElementsByTagNameNS($ANP_NAMESPACE, $REDIRECT_URL_ELEMENT_NAME);

	#TODO - A few literals here
	my $libertyUserName = $policyEngine
	    ->lookupSecureInfo($LOGIN_INFO_USER_INFO_TYPE . '/' . $USER_NAME_USER_INFO_NAME,
			       $selectedIDProvideDomainString);
	       
	my $libertyPassword = $policyEngine
	    ->lookupSecureInfo($LOGIN_INFO_USER_INFO_TYPE . '/' . $PASSWORD_USER_INFO_NAME,
			       $selectedIDProviderDomainString);

	if ($customValues{$LOGIN_INFO_USER_INFO_TYPE . '/' . $USER_NAME_USER_INFO_NAME})
	{
	    $libertyUserName = $customValues{$LOGIN_INFO_USER_INFO_TYPE . '/' . $USER_NAME_USER_INFO_NAME};
	}
	if ($customValues{$LOGIN_INFO_USER_INFO_TYPE . '/' . $PASSWORD_USER_INFO_NAME})
	{
	    $libertyPassword = $customValues{$LOGIN_INFO_USER_INFO_TYPE . '/' . $PASSWORD_USER_INFO_NAME};
	}
	
	if ($PRINT_INFORMATION)
	{
	    print(STDERR $PRINT_INFORMATION_PREFIX . $PRINT_LOCAL_PREFIX . "Entering Liberty authentication\n");
	    if ($IDProviderRedirectURL)
	    {
		print(STDERR $PRINT_INFORMATION_PREFIX . $PRINT_LOCAL_PREFIX . "RedirectURL = " .
		      $IDProviderRedirectURL->textContent . "\n");
	    }
	    print(STDERR $PRINT_INFORMATION_PREFIX . $PRINT_LOCAL_PREFIX . "Username = " . $libertyUserName .
		  ", password = " . $libertyPassword . "\n");
	}
	
	
	
	# If a redirect URL is given, continue with extended Liberty authentication
	if ($IDProviderRedirectURL)
	{
	    if ($MEASURING_COMPONENTS)
	    {
		push(@$measurementDescriptions, "140. Entering Liberty extended auth");
		push(@$measurementTimes, gettimeofday());
	    }

	    return $this->libertyExtendedAuthentication($IDProviderRedirectURL->textContent,
						  $libertyUserName, $libertyPassword,
						  $authQueryKey, $MACAddress,
						  $querySOAPWithCookies, $requestSOAPMethod);
	}
	else
	{
	    if ($PRINT_ERROR)
	    {
		print(STDERR $PRINT_ERROR_PREFIX . $PRINT_LOCAL_PREFIX . "Couldnt extract ID Provider RedirectURL\n");
	    }

	    if ($MEASURING_COMPONENTS)
	    {
		push(@$measurementDescriptions, "150. Exiting");
		push(@$measurementTimes, gettimeofday());
	    }

	    return $FAILURE_AUTH_RESULT;
	}
    }
    # In any other case (An unkown value of AuthnResult, both AuthnResult and StatusCode are missing, etc)
    #   we assume a failure.
    else
    {
	if ($PRINT_ERROR)
	{
	    print(STDERR $PRINT_ERROR_PREFIX . $PRINT_LOCAL_PREFIX .
		  "Authentication failed because normal AuthnResult and StatusCode values were not seen\n");
	}

	if ($MEASURING_COMPONENTS)
	{
	    push(@$measurementDescriptions, "160. Exiting");
	    push(@$measurementTimes, gettimeofday());
	}

	return $FAILURE_AUTH_RESULT;
    }
}

# Given the text of an AuthnStatement, determineAuthOutcome will return (Not exit the program, printing the outcome)
#   only if a definite outcome could not be determined. In such a case, code following the call to determineAuthOutcome
#   is responsible for performing further analysis in order to determine this programs outcome.
sub determineAuthOutcome
{
    my $PRINT_LOCAL_PREFIX = "determineAuthOutcome() - ";

    my $this = shift;
    my ($authStatement) = @_;

    my $authStatementParser = XML::LibXML->new();
    
    my $authStatementParseTree = $authStatementParser->parse_string($authStatement);

    my ($authResult) = $authStatementParseTree->getElementsByTagNameNS($ANP_NAMESPACE, $AUTH_RESULT_ELEMENT_NAME);
    # If an AuthnResult element exists, look at its value.
    if ($authResult)
    {
	my $authResultValue = $authResult->textContent;

	# If its value is one of "Successful" or "Failed", then return that as our auth result.
	if ($authResultValue eq $SUCCESS_AUTH_RESULT)
	{
	    if ($PRINT_INFORMATION)
	    {
		print(STDERR $PRINT_INFORMATION_PREFIX . $PRINT_LOCAL_PREFIX .
		      "Auth succeeded. Found AuthnResult element with value " .
		      $authResultValue . "\n");
	    }	    

	    return $SUCCESS_AUTH_RESULT;
	}
	elsif ($authResultValue eq $FAILURE_AUTH_RESULT)
	{
	    if ($PRINT_INFORMATION)
	    {
		print(STDERR $PRINT_INFORMATION_PREFIX . $PRINT_LOCAL_PREFIX .
		      "Auth failed. Found AuthnResult element with value " .
		      $authResultValue . "\n");
	    }

	    return $FAILURE_AUTH_RESULT;
	}
    }
    # Otherwise, look for a StatusCode element
    else
    {
	my ($statusCode) = $authStatementParseTree
	    ->getElementsByTagNameNS($SAMLP_NAMESPACE, $AUTH_STATUS_CODE_ELEMENT_NAME);

	# If it exists, then auth succeeds if its "Value" attribute equals "saml:Success"
	if ($statusCode)
	{
	    my $statusCodeValue = $statusCode->getAttribute($AUTH_STATUS_CODE_VALUE_ATTRIBUTE_NAME);

	    if ($PRINT_INFORMATION)
	    {
		print(STDERR $PRINT_INFORMATION_PREFIX . $PRINT_LOCAL_PREFIX . "Found StatusCode element with value " .
		      $statusCodeValue . "\n");
	    }

	    if ($statusCodeValue eq $SUCCESSFUL_STATUS_CODE)
	    {
		return $SUCCESS_AUTH_RESULT;
	    }
	    # Otherwise, it fails
	    else
	    {
		return $FAILURE_AUTH_RESULT;
	    }
	}
    }

    return "";
}



# Given the ID Provider redirect URL (From the RedirectURL of the AuthStatement) and the userName and password
#   (From the secureConf file), libertyExtendedAuthentication will perform the necessary additional
#   authentication steps associated with using Liberty.
sub libertyExtendedAuthentication
{
    my $PRINT_LOCAL_PREFIX = 'libertyExtendedAuthentication() - ';

    my $this = shift;

    my $measurementDescriptions;
    my $measurementTimes;
    if ($MEASURING_COMPONENTS)
    {
	$measurementDescriptions = $this->{$MEASUREMENT_DESCRIPTIONS_KEY};
	$measurementTimes = $this->{$MEASUREMENT_TIMES_KEY};

	push(@$measurementDescriptions, "140.10 Liberty auth -> Entering libertyExtendedAuth");
	push(@$measurementTimes, gettimeofday());
    }

    my ($IDProviderRedirectURL, $userName, $password, $authQueryKey, $MACAddress,
	$querySOAPWithCookies, $requestSOAPMethod) = @_;

    my $ID_PROVIDER_AUTH_REQUEST_CONTENT_TYPE = 'application/x-www-form-urlencoded';

    my $REDIRECT_HEADER_NAME = 'Location:';

    my $authRequestUserNameComponent = '_authn_name=' . $userName;
    my $authRequestPasswordComponent = '_authn_pass=' . $password;
    my $AUTH_REQUEST_LIBERTY_ID_COMPONENT = '_liberty_id=_liberty_local';
    my $ID_PROVIDER_AUTH_REQUEST_CONTENT = $authRequestUserNameComponent . '&' .
	$authRequestPasswordComponent . '&' . $AUTH_REQUEST_LIBERTY_ID_COMPONENT;




    my $IDProviderCookies = HTTP::Cookies->new();
    my $webRequester = LWP::UserAgent->new(cookie_jar => $IDProviderCookies);


    if ($MEASURING_COMPONENTS)
    {
	push(@$measurementDescriptions, "140.20 Liberty auth -> About to send GET to ID Provider Redirect URL");
	push(@$measurementTimes, gettimeofday());
    }

    # "Go" to the RedirectURL (The ID Provider) specified in the AuthResponse
    my $IDProviderRedirectRequest = HTTP::Request->new(GET => $IDProviderRedirectURL);
    my $IDProviderResponse = $webRequester->simple_request($IDProviderRedirectRequest);

    if ($MEASURING_COMPONENTS)
    {
	push(@$measurementDescriptions, "140.30 Liberty auth -> Done sending GET");
	push(@$measurementTimes, gettimeofday());
    }

    if ($PRINT_MSGS)
    {
	print (STDERR $IDProviderResponse . "\n");
    }

    $IDProviderCookies->extract_cookies($IDProviderResponse);



    if ($IDProviderRedirectURL)
    {
	if ($PRINT_INFORMATION)
	{
	    print(STDERR $PRINT_INFORMATION_PREFIX . $PRINT_LOCAL_PREFIX .
		  "Posting ID provider auth to " . $IDProviderRedirectURL . "\n");
	}

	# Build and send the authentication response to the ID provider
	$IDProviderAuthRequest = HTTP::Request->new(POST => $IDProviderRedirectURL);
	$IDProviderAuthRequest->content_type($ID_PROVIDER_AUTH_REQUEST_CONTENT_TYPE);
	$IDProviderAuthRequest->content($ID_PROVIDER_AUTH_REQUEST_CONTENT);
	$IDProviderCookies->add_cookie_header($IDProviderAuthRequest);

	if ($MEASURING_COMPONENTS)
	{
	    push(@$measurementDescriptions, "140.40 Liberty auth -> About to send POST to ID Provider Redirect URL");
	    push(@$measurementTimes, gettimeofday());
	}
	
	$IDProviderAuthResponse = $webRequester->simple_request($IDProviderAuthRequest);

	if ($MEASURING_COMPONENTS)
	{
	    push(@$measurementDescriptions, "140.50 Liberty auth -> Done sending POST");
	    push(@$measurementTimes, gettimeofday());
	}

	if ($PRINT_MSGS)
	{
	    print (STDERR $IDProviderAuthResponse . "\n");
	}

	# Extract the service provider redirect url
	my $serviceProviderRedirectURL = $this->extractHeaderValue($IDProviderAuthResponse, $REDIRECT_HEADER_NAME);
	
	
	
	if ($serviceProviderRedirectURL)
	{
	    if ($PRINT_INFORMATION)
	    {
		print(STDERR $PRINT_INFORMATION_PREFIX . $PRINT_LOCAL_PREFIX . "Service provider redirect URL = " .
		      $serviceProviderRedirectURL . "\n");
		
	    }

	    if ($MEASURING_COMPONENTS)
	    {
		push(@$measurementDescriptions, "140.60 Liberty auth -> Starting to build auth continuation query");
		push(@$measurementTimes, gettimeofday());
	    }

	    # Build a new SOAP query containing the Service Provider redirect URL (Just extracted from the
	    #   ID Provider auth response), and the auth key and MAC address (Taken as arguments to this program)
	    my @authContQueryInfo;

	    push(@authContQueryInfo,
		 SOAP::Data->name($REDIRECTED_URL_ELEMENT_NAME => $serviceProviderRedirectURL)
		 ->prefix($ANP_PREFIX));
	    push(@authContQueryInfo,
		 SOAP::Data->name($USER_INFO_ELEMENT_NAME =>
				  \SOAP::Data->name($USER_INFO_ATTRIBUTE_VALUE_ATTRIBUTE_NAME => $authQueryKey)
				  ->prefix($SAML_PREFIX)
				  ->uri($SAML_NAMESPACE))
		 ->prefix($ANP_PREFIX)
		 ->attr({AttributeName => $USER_INFO_AUTH_QUERY_KEY_ATTRIBUTE_NAME}));
	    push(@authContQueryInfo,
		 SOAP::Data->name($USER_INFO_ELEMENT_NAME =>
				  \SOAP::Data->name($USER_INFO_ATTRIBUTE_VALUE_ATTRIBUTE_NAME => $MACAddress)
				  ->prefix($SAML_PREFIX)
				  ->uri($SAML_NAMESPACE))
		 ->prefix($ANP_PREFIX)
		 ->attr({AttributeName => $USER_INFO_MAC_ADDRESS_ATTRIBUTE_NAME}));

	    # Send that query as an AuthnContinuationQuery
 	    my $authContQuery =  SOAP::Data->name($AUTH_CONT_QUERY_ELEMENT_NAME => 
						     \SOAP::Data->value(@authContQueryInfo)
						     ->prefix($ANP_PREFIX));

	    if ($MEASURING_COMPONENTS)
	    {
		push(@$measurementDescriptions, "140.70 Liberty auth -> Done building cont query. About to send it");
		push(@$measurementTimes, gettimeofday());
	    }

	    my $libertyAuthStatement = $querySOAPWithCookies->call($requestSOAPMethod => $authContQuery);

	    if ($MEASURING_COMPONENTS)
	    {
		push(@$measurementDescriptions, "140.80 Liberty auth -> Done sending cont query");
		push(@$measurementTimes, gettimeofday());
	    }

	    if ($PRINT_MSGS)
	    {
		print (STDERR $libertyAuthStatement . "\n");
	    }

	    # Analyze the result in the same way we analyzed the original AuthnResult during non-Liberty authentication

	    my $authOutcome = $this->determineAuthOutcome($libertyAuthStatement);
	    if (($authOutcome eq $SUCCESS_AUTH_RESULT) || ($authOutcome eq $FAILURE_AUTH_RESULT))
	    {
		if ($MEASURING_COMPONENTS)
		{
		    push(@$measurementDescriptions, "140.90 Liberty auth -> Returning");
		    push(@$measurementTimes, gettimeofday());
		}

		return $authOutcome;
	    }
	    

	    # However, in this case, there is no extended analysis to be done; If the liberty AuthnStatement doesnt
	    #   contain a definite success or fail outcome, then we default to failure.
	    if ($PRINT_ERROR)
	    {
		print(STDERR $PRINT_ERROR_PREFIX . $PRINT_LOCAL_PREFIX .
		      "Didnt get conclusive success or failure during authentication. Defaulting to failure.\n");
	    }

	    if ($MEASURING_COMPONENTS)
	    {
		push(@$measurementDescriptions, "140.100 Liberty auth -> Returning");
		push(@$measurementTimes, gettimeofday());
	    }

	    return $FAILURE_AUTH_RESULT;
	}
	else
	{
	    if ($PRINT_ERROR)
	    {
		print(STDERR $PRINT_ERROR_PREFIX . $PRINT_LOCAL_PREFIX . "Invalid service provider redirect URL\n");
	    }
	}
    }
    else
    {
	if ($PRINT_ERROR)
	{
	    print(STDERR $PRINT_ERROR_PREFIX . $PRINT_LOCAL_PREFIX . "Invalid ID provider post Auth URL\n");
	}
    }

    if ($MEASURING_COMPONENTS)
    {
	push(@$measurementDescriptions, "140.110 - Liberty auth -> Returning");
	push(@$measurementTimes, gettimeofday());
    }

    return $FAILURE_AUTH_RESULT;
}

# Given an HTTP::Response and a string describing the desired Header name to look for,
# extractHeaderValue will return the value associated with that header from that response.
sub extractHeaderValue
{
    my $PRINT_LOCAL_PREFIX = 'extractHeaderValue() - ';



    my $this = shift;
    my ($response, $desiredHeaderNameString) = @_;

    my $responseHeaderBlock = $response->headers->as_string;



    if ($PRINT_INFORMATION)
    {
	print(STDERR $PRINT_INFORMATION_PREFIX . $PRINT_LOCAL_PREFIX . "Looking for header \'" .
	      $desiredHeaderNameString . "\' in header block :\n" . $responseHeaderBlock);
    }

    my @responseHeaders = split (/\n/, $responseHeaderBlock);
    foreach my $currentResponseHeader (@responseHeaders)
    {
        my @headerComponents = split(/ /, $currentResponseHeader);
        
	if ($headerComponents[$HEADER_NAME_COMPONENT_INDEX] eq $desiredHeaderNameString)
	{
	    return $headerComponents[$HEADER_VALUE_COMPONENT_INDEX];
        }
    }

    return "";
}

# 'use ANPClient' requires a true return value when loading the module.
return $TRUE;
