/*
 * $Id: LibertyAssertion.java,v 1.8.2.1 2002/09/17 01:52:21 jsergent Exp $
 * LibertyAssertion.java
 *
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.experimentalstuff.com/Technologies/IPL/download.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2002 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 */

package com.sun.liberty;
import com.sun.liberty.saml.SAMLAssertion;
import java.util.Iterator;
import java.util.List;
import java.util.LinkedList;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * Data binding for lib:Assertion, the Liberty extensions to the
 * SAML Assertion element.
 * 
 * @author Jonathan Sergent
 * @version $Revision: 1.8.2.1 $
 */
public class LibertyAssertion extends SAMLAssertion implements LibertyElement {
    /** Storage for InResponseTo attribute */
    private String inResponseTo;

    /**
     * Parse XML element to populate fields.
     * @param e Element containing a lib:Assertion or extension thereof
     */
    public void parse(Element e) {
	setInResponseTo(e.getAttributeNS(null, "InResponseTo"));
	NodeList list = e.getChildNodes();
	
	for (int j = 0; j < list.getLength(); j++) {
	    if (list.item(j) instanceof org.w3c.dom.Text) { continue; }
	    Element child = (Element) list.item(j);
	    if (isElement(child, LIB_NS_URI, "AuthenticationStatement")) {
		LibertyAuthenticationStatement c = new LibertyAuthenticationStatement();
		c.parse(child);
		List set = getStatements();
		if (set == null) {
		    set = new LinkedList();
		}
		set.add(c);
		setStatements(set);
	    }
	}
	super.parse(e);
    }
    
    public Element getElement(Document doc) {
	Element r = doc.createElementNS(SAML_NS_URI, "saml:Assertion");
	r.setAttributeNS(XSI_NS_URI, "xsi:type", "lib:AssertionType");
	addAttributesTo(r);
	addChildrenTo(r);
	return r;
    }

    /** 
     * Adds attributes from this class to the element,
     * then calls superclass addAttributesTo.
     * @param s The element
     */
    protected void addAttributesTo(Element s) {
	if (inResponseTo != null) {
	    s.setAttributeNS(null, "InResponseTo", inResponseTo);
	}
	super.addAttributesTo(s);
    }

    /** 
     * Adds child elements from this class to the element,
     * then calls superclass addAttributesTo.  (Note: No child elements
     * from this class.)
     * @param s The element
     */
    protected void addChildrenTo(Element s) {
	super.addChildrenTo(s);
    }

    /**
     * Returns value of InResponseTo attribute.  Contains RequestID
     * of the request that was sent for this assertion.
     * @return String containing request ID value
     */
    public String getInResponseTo() {
	return inResponseTo;
    }

    /**
     * Sets value of InResponseTo attribute.  Should contain RequestID
     * of the request that was sent for this assertion.
     * @param value String containing request ID value
     */
    public void setInResponseTo(String value) {
	inResponseTo = value;
    }
}
