/*
 * $Id: LibertyAuthenticationStatement.java,v 1.6.2.1 2002/09/17 01:52:22 jsergent Exp $
 * LibertyAuthenticationStatement.java
 *
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.experimentalstuff.com/Technologies/IPL/download.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2002 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 */

package com.sun.liberty;
import com.sun.liberty.saml.SAMLAuthenticationStatement;
import com.sun.liberty.ac.LibertyAuthenticationContext;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * Data binding for Liberty extensions to saml:AuthenticationStatement.
 * NOTE: AuthenticationContext is not yet implemented (stubs only).
 * @author Jonathan Sergent
 * @version $Revision: 1.6.2.1 $
 */
public class LibertyAuthenticationStatement extends SAMLAuthenticationStatement implements LibertyElement {
    /** Storage for AuthenticationContext child element. */
    private LibertyAuthenticationContext authenticationContext;
    /** Storage for ReauthenticateOnOrAfter attribute */
    private Date reauthenticateOnOrAfter;
    /** Storage for SessionIndex attribute */
    private String sessionIndex;


    /**
     * Parse XML element to populate fields.
     * @param e Element of type lib:AuthenticationStatementType
     */
    public void parse(Element e) {
	super.parse(e);
	setReauthenticateOnOrAfter(parseDate(e.getAttributeNS(null, "ReauthenticateOnOrAfter")));
	setSessionIndex(e.getAttributeNS(null, "SessionIndex"));
	NodeList list = e.getChildNodes();
	for (int j = 0; j < list.getLength(); j++) {
	    if (list.item(j) instanceof org.w3c.dom.Text) { continue; }
	    Element child = (Element) list.item(j);
	    if (isElement(child, AUTHCTX_NS_URI, "AuthenticationContext")) {
//		LibertyAuthenticationContext c = new LibertyAuthenticationContext();
//		c.parse(e);
//		setAuthenticationContext(c);
	    }
	}
    }

    public Element getElement(Document doc) {
	Element r = doc.createElementNS(SAML_NS_URI,
				 "saml:AuthenticationStatement");
	r.setAttributeNS(XSI_NS_URI, "xsi:type",
		  "lib:AuthenticationStatementType");
	addAttributesTo(r);
	addChildrenTo(r);
	return r;
    }

    /** 
     * Adds attributes from this class to the element,
     * then calls superclass addAttributesTo.
     * @param s The element
     */
    protected void addAttributesTo(Element s) {
	if (reauthenticateOnOrAfter != null) {
	    s.setAttributeNS(null, "ReauthenticateOnOrAfter",
		      formatDate(reauthenticateOnOrAfter));
	}
	if (sessionIndex != null) {
	    s.setAttributeNS(null, "SessionIndex", sessionIndex);
	}
	
	super.addAttributesTo(s);
    }

    /** 
     * Adds child elements from this class to the element,
     * then calls superclass addChildrenTo.
     * @param s The element
     */
    protected void addChildrenTo(Element s) {
	if (authenticationContext != null) {
	    s.appendChild(authenticationContext.getElement(s.getOwnerDocument()));
	}
	
	super.addChildrenTo(s);
    }

    /**
     * Returns AuthenticationContext child element if present
     * @return the element, or null if not present
     */
    public LibertyAuthenticationContext getAuthenticationContext() {
	return authenticationContext;
    }

    /**
     * Sets or clears the AuthenticationContext child element
     * @param value the element, or null
     */
    public void setAuthenticationContext(LibertyAuthenticationContext value) {
	authenticationContext = value;
    }

    /**
     * Returns ReauthenticateOnOrAfter time if present
     * @return the Date object, or null if not present
     */
    public Date getReauthenticateOnOrAfter() {
	return reauthenticateOnOrAfter;
    }

    /**
     * Sets or clears the ReauthenticateOnOrAfter time
     * @param value the Date object, or null
     */
    public void setReauthenticateOnOrAfter(Date value) {
	reauthenticateOnOrAfter = value;
    }

    /**
     * Returns the session index if present
     * @return the session index, or null if not present
     */
    public String getSessionIndex() {
	return sessionIndex;
    }

    /**
     * Sets or clears the session index.
     * @param value the index, or null
     */
    public void setSessionIndex(String value) {
	sessionIndex = value;
    }
}
