/*
 * $Id: LibertyAuthnRequest.java,v 1.8.2.1 2002/09/17 01:52:22 jsergent Exp $
 * LibertyAuthnRequest.java
 *
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.experimentalstuff.com/Technologies/IPL/download.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2002 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 */

package com.sun.liberty;
import com.sun.liberty.saml.SAMLPAbstractRequest;
import java.util.Iterator;
import java.util.List;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * Data bindings for the lib:AuthnRequest element.
 * Note that it is usually desirable to instantiate one of the subclasses
 * of this class that implements a particular Liberty binding or profile.
 *
 * @author Jonathan Sergent
 * @version $Revision: 1.8.2.1 $
 */
public class LibertyAuthnRequest extends SAMLPAbstractRequest implements LibertyElement {
    /** Storage for the ProviderID */
    private String providerID;
    /** Storage for the IsPassive flag */
    private boolean isPassive = true;
    /** Storage for the ForceAuthn flag */
    private boolean forceAuthn = false;
    /** Storage for the Federate flag */
    private boolean federate = false;
    /** Storage for the ProtocolProfile value */
    private LibertyProfile protocolProfile = LibertyProfile.BRWS_ART;
    /** Storage for the AuthnContext element */
    private LibertyRequestedAuthnContext authnContext = null;
    /** Storage for the RelayState */
    private String relayState = null;

    public Element getElement(Document doc) {
	Element r = doc.createElementNS(LIB_NS_URI, "lib:AuthnRequest");
	addAttributesTo(r);
	addChildrenTo(r);
	return r;
    }

    /** 
     * Calls superclass addAttributesTo.
     * @param r The element
     */
    protected void addAttributesTo(Element r) {
	super.addAttributesTo(r);
    }

    /** 
     * Adds child elements from this class to the element,
     * then calls superclass addChildrenTo.
     * @param s The element
     */
    protected void addChildrenTo(Element r) {
	appendNewChildNSWithText(r, LIB_NS_URI, "lib:ProviderID", providerID);
	appendNewChildNSWithText(r, LIB_NS_URI, "lib:IsPassive",
				 new Boolean(isPassive).toString());
	appendNewChildNSWithText(r, LIB_NS_URI, "lib:ForceAuthn",
				 new Boolean(forceAuthn).toString());
	appendNewChildNSWithText(r, LIB_NS_URI, "lib:Federate",
				 new Boolean(federate).toString());
	appendNewChildNSWithText(r, LIB_NS_URI, "lib:ProtocolProfile",
				 protocolProfile.toString());
	if (authnContext != null) {
	    r.appendChild(authnContext.getElement(r.getOwnerDocument()));
	}
	appendNewChildNSWithText(r, LIB_NS_URI, "lib:RelayState", relayState);
	super.addChildrenTo(r);
    }

    // XXX: Need parse(Element) for LECP

    /**
     * Returns contents of ProviderID child, if present.
     * @return the URI in a string, or null if not present.
     */
    public String getProviderID() {
	return providerID;
    }

    /**
     * Sets or clears contents of ProviderID child.
     * Note: Also notes this provider ID for use in signature validation
     * of any future calls to parse().
     * @param value the URI in a string, or null if not present.
     */
    public void setProviderID(String value) {
	providerID = value;
	setReceivedFrom(value);
    }

    /**
     * Returns value of IsPassive flag.
     * If true, IDP will only process the request if it can be processed
     * without user interaction.
     * @return the flag value
     */
    public boolean getIsPassive() {
	return isPassive;
    }

    /**
     * Sets value of IsPassive flag.
     * If true, IDP will only process the request if it can be processed
     * without user interaction.
     * @param value the new value of the flag
     */
    public void setIsPassive(boolean value) {
	isPassive = value;
    }

    /**
     * Returns value of ForceAuthn flag.
     * If true, IDP will attempt to re-authenticate the user, if applicable,
     * even if the user already has an authenticated session.
     * @return the flag value
     */
    public boolean getForceAuthn() {
	return forceAuthn;
    }

    /**
     * Sets value of ForceAuthn flag.
     * If true, IDP will attempt to re-authenticate the user, if applicable,
     * even if the user already has an authenticated session.
     * @param value the new value of the flag
     */
    public void setForceAuthn(boolean value) {
	forceAuthn = value;
    }

    /**
     * Returns value of Federate flag.
     * If true, IDP will attempt federation upon receiving this request
     * if the user consents.
     * If false, AuthnRequest will fail unless an existing name identifier
     * mapping exists.
     * @return the flag value
     */
    public boolean getFederate() {
	return federate;
    }

    /**
     * Sets value of Federate flag.
     * If true, IDP will attempt federation upon receiving this request
     * if the user consents.
     * If false, AuthnRequest will fail unless an existing name identifier
     * mapping exists.
     * @param value the new value of the flag
     */
    public void setFederate(boolean value) {
	federate = value;
    }

    /**
     * Gets value of ProtocolProfile parameter.  The SP uses this field
     * to request a particular profile for the response from the IDP.
     * @return The selected profile.
     */
    public LibertyProfile getProtocolProfile() {
	return protocolProfile;
    }

    /**
     * Sets value of ProtocolProfile parameter.  The SP uses this field
     * to request a particular profile for the response from the IDP.
     * @param value The selected profile.
     */
    public void setProtocolProfile(LibertyProfile value) {
	protocolProfile = value;
    }

    /**
     * Gets the AuthnContext child element, if present.
     * @return The AuthnContext element, or null if not present.
     */
    public LibertyRequestedAuthnContext getAuthnContext() {
	return authnContext;
    }

    /**
     * Sets or clears the AuthnContext child element.  Used by the SP
     * to indicate authentication context requirements to the IDP.
     * @param value The requested authentication context, or null if none.
     */
    public void setAuthnContext(LibertyRequestedAuthnContext value) {
	authnContext = value;
    }

    /**
     * Gets the value of the RelayState child element, if present.
     * @return the relay state string, or null if not present.
     */
    public String getRelayState() {
	return relayState;
    }

    /**
     * Sets or clears the value of the RelayState child element.  Used
     * by the SP to save state between the AuthnRequest and the subsequent
     * assertion.  Can be used if the SP does not have cookie-based sessions
     * so that when the user agent is redirected back from the IDP, the
     * SP can track the session that the assertion is from.
     * @param value The relay state.
     */
    public void setRelayState(String value) {
	relayState = value;
    }
}
