/*
 * $Id: LibertySubject.java,v 1.6.2.1 2002/09/17 01:52:24 jsergent Exp $
 * LibertySubject.java
 *
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.experimentalstuff.com/Technologies/IPL/download.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2002 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 */

package com.sun.liberty;
import com.sun.liberty.saml.SAMLElement;
import com.sun.liberty.saml.SAMLSubject;
import java.util.Iterator;
import java.util.List;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * Data bindings for Liberty extensions to saml:Subject element.
 *
 * @author Jonathan Sergent
 * @version $Revision: 1.6.2.1 $
 */
public class LibertySubject extends SAMLSubject implements LibertyElement {
    LibertyIDPProvidedNameIdentifier idpProvidedNameID = null;

    public void parse(Element e) {
	super.parse(e);
	NodeList list = e.getChildNodes();
	for (int j = 0; j < list.getLength(); j++) {
	    if (list.item(j) instanceof org.w3c.dom.Text) { continue; }
	    Element child = (Element) list.item(j);
	    if (isElement(child, LIB_NS_URI, "IDPProvidedNameIdentifier")) {
		LibertyIDPProvidedNameIdentifier c = new LibertyIDPProvidedNameIdentifier();
		c.parse(e);
		setIDPProvidedNameIdentifier(c);
	    }
	}
    }
		
    public Element getElement(Document doc) {
	/*
	 * There is probably a more correct way to handle XML Schema
	 * types than this.
	 */
	Element r = doc.createElementNS(SAMLElement.SAML_NS_URI,
				 SAMLElement.SAML_NS_ID + ":Subject");
	r.setAttributeNS(XSI_NS_URI, "xsi:type", "lib:SubjectType");
	addChildrenTo(r);
	addAttributesTo(r);
	return r;
    }

    protected void addAttributesTo(Element s) {
    }
    
    protected void addChildrenTo(Element s) {
	if (idpProvidedNameID != null) {
	    s.appendChild(idpProvidedNameID.getElement(s.getOwnerDocument()));
	}
	super.addChildrenTo(s);
    }

    /**
     * Gets the IDP-provided name identifier.  If a name identifier has
     * been registered by the SP using the RegisterNameIdentifier request,
     * this continues to carry the IDP-provided name in messages from the
     * IDP to the SP.  (Other messages, such as logout notifications,
     * only carry the identifier provided by the SP in this situation.)
     *
     * @return the IDP-provided name identifier.
     */
    public LibertyIDPProvidedNameIdentifier getIDPProvidedNameIdentifer() {
	return idpProvidedNameID;
    }


    /**
     * Sets the IDP-provided name identifier.  If a name identifier has
     * been registered by the SP using the RegisterNameIdentifier request,
     * this continues to carry the IDP-provided name in messages from the
     * IDP to the SP.  (Other messages, such as logout notifications,
     * only carry the identifier provided by the SP in this situation.)
     * Otherwise, it should be the same as the NameIdentifier.
     *
     * @param value the IDP-provided name identifier
     */
    public void setIDPProvidedNameIdentifier(LibertyIDPProvidedNameIdentifier value) {
	idpProvidedNameID = value;
    }
}
