/*
 * $Id: LibertyURLMessage.java,v 1.4.2.1 2002/09/17 01:52:25 jsergent Exp $
 * LibertyURLMessage.java
 *
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.experimentalstuff.com/Technologies/IPL/download.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2002 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 */

package com.sun.liberty;

/**
 * Common interface for messages sent using the URL query component.
 *
 * @author Jonathan Sergent
 * @version $Revision: 1.4.2.1 $
 */
public interface LibertyURLMessage {
    /**
     * Formats the message into a URL query string.
     *
     * @return  String suitable for appending to a URL or including in a
     *          POST body.
     *          Does not including a leading "?".
     */    
    public String getQuery() throws Exception;

    /**
     * Parses the given URL query string and updates the object to match.
     * Note that previously set values may persist if the given query does not
     * contain values for the fields, so it is usually correct to only call this
     * method on a newly created object.
     *
     * May throw an exception if the request cannot be parsed or it contains
     * invalid data (including a signature that can't be verified).
     *
     * @param	query	The query string containing the request.
     */
    public void parse(String query) throws Exception;

    /**
     * Formats the message into an HTML fragment which will cause the
     * user agent to POST the proper URL query string.
     *
     * @param	url	The URL to POST to
     * @return  An HTML fragment (a <FORM> tag and the contents thereof)
     */
    public String getHTMLFragment(String url) throws Exception;
}
