/*
 * $Id: LibertyUser.java,v 1.10.2.2 2002/09/17 01:52:25 jsergent Exp $
 * LibertyUser.java
 *
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.experimentalstuff.com/Technologies/IPL/download.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2002 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 */

package com.sun.liberty;
import com.sun.liberty.saml.SAMLNameIdentifier;
import com.sun.security.auth.SolarisPrincipal;
import com.sun.security.auth.UnixPrincipal;
import java.lang.ref.WeakReference;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;
import java.util.StringTokenizer;
import javax.naming.Context;
import javax.naming.NamingEnumeration;
import javax.naming.directory.Attribute;
import javax.naming.directory.Attributes;
import javax.naming.directory.BasicAttribute;
import javax.naming.directory.BasicAttributes;
import javax.naming.directory.DirContext;
import javax.naming.directory.InitialDirContext;
import javax.naming.directory.SearchResult;
import javax.security.auth.Subject;

/**
 * Directory based Liberty user metadata.
 * XXX: get*Name* should return SAMLNameIdentifier instead of Name.
 * 
 * @author Jonathan Sergent
 * @version $Revision: 1.10.2.2 $
 */
public class LibertyUser {
    private static DirContext ctx;
    private static HashMap users = new HashMap();
    private WeakReference currentSession = null;
    private String name;

    static {
	java.util.Hashtable env = new java.util.Hashtable();
	env.put(Context.INITIAL_CONTEXT_FACTORY, Config.userMetadataICF);
	env.put(Context.PROVIDER_URL, Config.userMetadataURL);
	env.put(Context.SECURITY_PRINCIPAL, Config.userMetadataPrin);
	env.put(Context.SECURITY_CREDENTIALS, Config.userMetadataCred);
	try {
	    ctx = new InitialDirContext(env);
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }

    private LibertyUser(String dnpart) throws Exception {
        users.put(dnpart, new WeakReference(this));
        name = dnpart;
    }

    public void finalize() {
        dropInstance(name);
    }
    
    private synchronized static void dropInstance(String name) {
        if (users.get(name) != null) {
            users.remove(name);
        }
    }

    private synchronized static LibertyUser getInstance(String dnpart)
														throws Exception {
        WeakReference user = (WeakReference) users.get(dnpart);
        if (user != null && user.get() != null) {
            return (LibertyUser) user.get();
        }
        return new LibertyUser(dnpart);
    }

    /**
     * Is this user (of the local IDP) federated with the given SP?
     * @param sp the SP's descriptor
     * @return true if the user is federated with sp
     */
    public boolean getFederationStatus(SPDescriptor sp) throws Exception {
        return (getLocalNameFor(sp) != null);
    }

    /**
     * Is this user (of the local SP) federated with the given IDP?
     * @param idp the IDP's descriptor
     * @return true if the user is federated with idp
     */
    public boolean getFederationStatus(IDPDescriptor idp) throws Exception {
        return (getLocalNameFor(idp) != null);
    }

    /**
     * Sets the federation status for the given SP for the user of
     * the local IDP.
     * setLocalNameFor() must be called after setting federation status 
     * to true.
     *
     * @param sp the SP to set federation for
     * @param value true to federate, false to terminate federation
     */
    public void setFederationStatus(SPDescriptor sp, boolean value) throws Exception {
	if (getFederationStatus(sp) == value) {
	    if (Config.debug) { System.err.println(value + " == " + value); }
	    return;
	}
	setFederationStatus(sp, "federatedServices", value);
    }


    /**
     * Sets the federation status for the given IDP for the user of the
     * local SP.
     * setLocalNameFor() and setNameProvidedBy() must be called after
     * setting federation status to true.
     *
     * @param idp the IDP to set federation for
     * @param value true to federate, false to terminate federation
     */
    public void setFederationStatus(IDPDescriptor idp, boolean value) throws Exception {
	if (getFederationStatus(idp) == value) {
	    if (Config.debug) { System.err.println(value + " == " + value); }
	    return;
	}
	setFederationStatus(idp, "federatedIdentities", value);
    }

    /** Common implementation of two-argument setFederationStatus() */
    private void setFederationStatus(ProviderDescriptor sp, String attrName, boolean value) throws Exception {
	Attributes attrs = new BasicAttributes();
	Attribute attr = new BasicAttribute(attrName);
	attrs.put(attr);

	if (value == false) {
	    attr.add(getFedEntry(attrName, sp));
	    ctx.modifyAttributes(name, DirContext.REMOVE_ATTRIBUTE, attrs);
	    if (Config.debug) { System.err.println("removed federatedServices attr value"); }
	} else {
	    attr.add(sp.getBase64ProviderSuccinctID() + "&&");
	    ctx.modifyAttributes(name, DirContext.ADD_ATTRIBUTE, attrs);
	    if (Config.debug) { System.err.println("added federatedServices attr value"); }
	}
    }

    /**
     * Gets the string value for the given SP of the user's federation
     * attribute.
     */
    private String getFedEntry(String attrName, ProviderDescriptor sp) throws Exception {
	if (Config.debug) { System.err.println("getFedEntry attrName:" + attrName); }
        Attributes attrs = ctx.getAttributes(name, new String[] { attrName });
        Attribute attr = attrs.get(attrName);
	if (attr != null) {
	    for (int i = 0; i < attr.size(); i++) {
		String val = (String) attr.get(i);
		if (Config.debug) { System.err.println("val <- " + val); }
		if (val.indexOf(sp.getBase64ProviderSuccinctID()) == 0) {
		    if (Config.debug) { System.err.println("matches"); }
		    return val;
		}
	    }
	}
        return null;        
    }

    /** Utility function to extract name from a federation attribute. */
    private SAMLNameIdentifier doGetName(String attrName,
					 ProviderDescriptor prov,
					 boolean local) throws Exception {
	String val = getFedEntry(attrName, prov);
	if (val == null) {
	    return null;
	} else {
	    // (val.indexOf(prov.getBase64ProviderSuccinctID()) == 0)
	    int j = val.indexOf("&");
	    int k = val.indexOf("&", j+1);
	    if (Config.debug) { System.err.println("val:" + val + " j:" + j + " k:" + k); }
	    SAMLNameIdentifier rv = new SAMLNameIdentifier();
	    if (local) {
		rv.parse(val.substring(k+1));
	    } else {
		rv.parse(val.substring(j+1, k));
	    }
	    return rv;
	}
    }

    /** Utility function to set a name in a federation attribute. */
    private void doSetName(String attrName, ProviderDescriptor prov, boolean local, SAMLNameIdentifier value) throws Exception {
        Attributes attrs = ctx.getAttributes(name, new String[] { attrName });
        Attribute attr = attrs.get(attrName);
        for (int i = 0; i < attr.size(); i++) {
            String val = (String) attr.get(i);
            if (val.indexOf(prov.getBase64ProviderSuccinctID()) == 0) {
		int j = val.indexOf("&");
		int k = val.indexOf("&", j+1);
		if (Config.debug) { System.err.println("val:" + val + " j:" + j + " k:" + k); }
                String newVal = val.substring(0, j+1);
                String oldRemote = val.substring(j+1, k);
                String oldLocal = val.substring(k+1);
		if (Config.debug) {
		    System.err.println("oldRemote:" + oldRemote
			 + " oldLocal:" + oldLocal);
		}
                if (!local) {
                    newVal += value.toString() + "&" + oldLocal;
                } else {
                    newVal += oldRemote + "&" + value.toString();
                }
		if (Config.debug) { System.err.println("newVal:" + newVal); }
                attr.set(i, newVal);
		ctx.modifyAttributes(name, DirContext.REPLACE_ATTRIBUTE, attrs);
		return;
            }
        }
        return;
    }

    /**
     * Gets the name provided by the given SP for use by the local IDP.
     *
     * @param sp The SP to use when looking for the name
     * @return The name, or null if not found
     */
    public SAMLNameIdentifier getNameProvidedBy(SPDescriptor sp) throws Exception {
	if (Config.debug) { System.err.println("getNameProvidedBy sp:" + sp.getProviderID()); }
        return doGetName("federatedServices", sp, false);
    }

    /**
     * Sets the name provided by the given SP for use by the local IDP.
     * To be called upon receiving a RegisterNameIdentifier request.
     *
     * @param sp The SP to set the name for
     * @param value The new name
     */
    public void setNameProvidedBy(SPDescriptor sp, SAMLNameIdentifier value) throws Exception {
	if (Config.debug) {
	    System.err.println("setNameProvidedBy sp:" + sp.getProviderID() + " value:" + value);
	}
        doSetName("federatedServices", sp, false, value);
    }

    /** 
     * Gets the name provided by the given IDP for use by the local SP.
     * To be used in messages addressed to the SP (logout, federation
     * termination, register name identifier).
     * @param idp The IDP to look up.
     * @return The name, or null if not found (i.e. not federated)
     */
    public SAMLNameIdentifier getNameProvidedBy(IDPDescriptor idp) throws Exception {
	if (Config.debug) {
	    System.err.println("getNameProvidedBy idp:" + idp.getProviderID());
	}
        return doGetName("federatedIdentities", idp, false);
    }

    /** 
     * Sets the name provided by the given IDP for use by the local SP.
     * To be called after successful federation occurs.
     * @param idp The IDP to set the name for
     * @param value The new name
     */
    public void setNameProvidedBy(IDPDescriptor idp, SAMLNameIdentifier value) throws Exception {
	if (Config.debug) {
	    System.err.println("setNameProvidedBy idp:" + idp.getProviderID() + " value:" + value);
	}
        doSetName("federatedIdentities", idp, false, value);
    }

    /**
     * Gets the name that the local IDP provided to the given SP.
     * @param sp The SP to use when looking for the name
     * @return The name, or null if not found
     */
    public SAMLNameIdentifier getLocalNameFor(SPDescriptor sp) throws Exception {
	if (Config.debug) {
	    System.err.println("getLocalNameFor sp:" + sp.getProviderID());
	}
        return doGetName("federatedServices", sp, true);
    }

    /** 
     * Sets the name that the local IDP provided to the given SP.
     * To be called when processing federation on the IDP.
     * @param idp The IDP to set the name for
     * @param value The new name
     */
    public void setLocalNameFor(SPDescriptor sp, SAMLNameIdentifier value) throws Exception {
	if (Config.debug) {
	    System.err.println("setLocalNameFor sp:" + sp.getProviderID() + " value:" + value);
	}
        doSetName("federatedServices", sp, true, value);
    }

    /**
     * Gets the name that the local SP provided to the given IDP.
     * @param idp The IDP to use when looking for the name
     * @return The name, or null if not found
     */
    public SAMLNameIdentifier getLocalNameFor(IDPDescriptor idp) throws Exception {
	if (Config.debug) {
	    System.err.println("getLocalNameFor idp:" + idp.getProviderID());
	}
	return doGetName("federatedIdentities", idp, true);
    }


    /** 
     * Sets the name that the local SP provided to the given IDP.
     * To be called after sending RegisterNameIdentifier to the IDP.
     * @param idp The IDP to set the name for
     * @param value The new name
     */
    public void setLocalNameFor(IDPDescriptor idp, SAMLNameIdentifier value) throws Exception {
	if (Config.debug) {
	    System.err.println("setLocalNameFor idp:" + idp.getProviderID() + " value:" + value);
	}
        doSetName("federatedIdentities", idp, true, value);
    }

    private static LibertyUser doFind(ProviderDescriptor prov, String attrName, SAMLNameIdentifier providedName) throws Exception {
        String filter;
        filter = "(" + attrName + "="
            + prov.getBase64ProviderSuccinctID() + "&"
	    + providedName.toString() + "&*)";
	if (Config.debug) { System.err.println("Filter is: " + filter); }

        NamingEnumeration en = ctx.search("", filter, null);
        if (en.hasMore()) {
            SearchResult sr = (SearchResult) en.next();
            return getInstance(sr.getName());
        } else {
	    if (Config.debug) { System.err.println("Search failed"); }
            return null;
        }
    }

    /** 
     * Finds a LibertyUser object with the name provided by an IDP.
     * Returns a singleton instance of LibertyUser for the user.
     * @param idp The IDP to search for
     * @param idpProvidedName The name provided by the IDP
     */
    public static LibertyUser find(IDPDescriptor idp, SAMLNameIdentifier idpProvidedName) throws Exception {
        return doFind(idp, "federatedIdentities", idpProvidedName);
    }

    /** 
     * Finds a LibertyUser object with the name provided by an SP.
     * Returns a singleton instance of LibertyUser for the user.
     * @param idp The SP to search for
     * @param idpProvidedName The name provided by the SP
     */
    public static LibertyUser find(SPDescriptor sp, SAMLNameIdentifier spProvidedName) throws Exception {
        return doFind(sp, "federatedServices", spProvidedName);
    }

    /**
     * Find LibertyUser based on Subject.
     * Returns a singleton instance of LibertyUser for the user.
     * This implementation will work with Subjects containing
     * the SolarisPrincipal principals from JndiLoginModule.
     * This should probably change to support more principals
     * or somehow be extensible so that JAAS login will work
     * with a variety of modules.
     */
    public static LibertyUser find(Subject subj) throws Exception {
	Set principals = subj.getPrincipals(SolarisPrincipal.class);
	for (Iterator i = principals.iterator(); i.hasNext(); ) {
	    SolarisPrincipal p = (SolarisPrincipal) i.next();
	    LibertyUser u = find(p.getName());
	    if (u != null) { return u; }
	}
	principals = subj.getPrincipals(UnixPrincipal.class);
	for (Iterator i = principals.iterator(); i.hasNext(); ) {
	    UnixPrincipal p = (UnixPrincipal) i.next();
	    LibertyUser u = find(p.getName());
	    if (u != null) { return u; }
	}
	return null;
    }

    /**
     * Find LibertyUser based on username.
     * Returns a singleton instance of LibertyUser for the user.
     * This implementation will work with usernames as generated by
     * LibertyUser.getUsername().
     */
    public static LibertyUser find(String username) throws Exception {
	String filter = "(uid=" + username + ")";
	NamingEnumeration en = ctx.search("", filter, null);
	if (en.hasMore()) {
	    SearchResult sr = (SearchResult) en.next();
	    return getInstance(sr.getName());
	}
	return null;
    }

    /**
     * Sets the current front channel session for the given user.
     * Can be retrieved later through getCurrentSession() to allow
     * SOAP-based logout.
     * @param s The user's session.  Should be a LibertyUserSession object.
     */
    public void setCurrentSession(Object s) {
        if (s != null) {
            currentSession = new WeakReference(s);
        } else {
            currentSession = null;
        }
    }

    /**
     * Returns the user's session as set previously with setCurrentSession.
     * @return The user's session.
     */
    public Object getCurrentSession() {
        return (currentSession == null) ? null : currentSession.get();
    }

    /**
     * Gets the local username associated with the LibertyUser object.
     * @return A string containing the username, or null if none was found.
     */
    public String getUsername() throws Exception {
        Attributes attrs = ctx.getAttributes(name, new String[] { "uid" });
        return (String) attrs.get("uid").get();
    }
}
