/*
 * $Id: ProviderDescriptor.java,v 1.5.2.1 2002/09/17 01:52:25 jsergent Exp $
 * ProviderDescriptor.java
 *
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.experimentalstuff.com/Technologies/IPL/download.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2002 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 */

package com.sun.liberty;

import java.security.PublicKey;
import java.net.URL;

/**
 * Stores provisioning information for a Liberty provider.
 * XXX: Should probably eventually provide interfaces for setting stuff, too.
 *
 * @author Jonathan Sergent
 * @version $Revision: 1.5.2.1 $
 */
public interface ProviderDescriptor {
    /**
     * Returns the provider ID (URI form).  Mandatory; will always be present.
     *
     * @return	The provider ID in URI form.
     */
    public String getProviderID();

    /**
     * Returns the provider ID in 160-bit binary form.  Mandatory; will always
     * be present.  This is a simple transform of the provider ID.
     *
     * @return	The provider ID in 160-bit binary form.
     */
    public byte[] getProviderSuccinctID();

    /**
     * Returns the providerSuccinctID encoded using BASE64.
     *
     * @return	The provider ID in 160-bit binary form, encoded with BASE64.
     */
    public String getBase64ProviderSuccinctID();

    /**
     * Returns the provider's public key, if available.
     * XXX: Should this be a certificate instead of a key?  The spec is not
     * clear.
     *
     * @return	Provider's public key.  Null if not present.
     */
    public PublicKey getKeyInfo();

    /**
     * Returns the provider's SOAP endpoint URL.
     *
     * @return	SOAP endpoint URL.  Null if not present.
     */
    public URL getSoapEndpoint();

    /**
     * Returns the provider's browser-based single logout URL.
     *
     * @return	Browser-based single logout URL.  Null if not present.
     */
    public URL getSingleLogoutURL();

    /**
     * Returns the provider's browser-based single logout return URL.
     *
     * @return	Browser-based single logout URL.  Null if not present.
     */
    public URL getSingleLogoutReturnURL();
    
    /**
     * Returns the provider's browser-based federation termination URL.
     *
     * @return	Browser-based fed term URL.  Null if not present.
     */
    public URL getFederationTerminationURL();

    /**
     * Returns the provider's browser-based federation termination return URL.
     *
     * @return	Browser-based fed term return URL.  Null if not present.
     */
    public URL getFederationTerminationReturnURL();
};
