/*
 * $Id: XMLUtils.java,v 1.18.2.3 2002/09/17 01:52:28 jsergent Exp $
 * XMLUtils.java
 *
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.experimentalstuff.com/Technologies/IPL/download.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2002 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): Ana Sanz Merino [University of California Berkeley, 2003]
 */

package com.sun.liberty;

import ucb.eecs.sahara.sso.DebugUtils;

import java.io.FileInputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.security.KeyStore;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Date;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import org.apache.xml.security.c14n.Canonicalizer;
import org.apache.xml.security.signature.XMLSignature;
import org.apache.xml.security.transforms.Transforms;
import org.apache.xml.security.transforms.params.XPathContainer;
import org.apache.xml.security.utils.Constants;
import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Collection of utilities for manipulating XML.
 * @author Jonathan Sergent
 * @version $Revision: 1.18.2.3 $
 */
public class XMLUtils {
    static DocumentBuilderFactory factory;
    static DocumentBuilder builder;
    static TransformerFactory tFactory;
    static Transformer transformer;
    static ProviderDescriptorFactory dir;

    static {
	try {
	    org.apache.xml.security.Init.init();
	    factory = DocumentBuilderFactory.newInstance();
	    factory.setNamespaceAware(true);
	    // factory.setIgnoringElementContentWhitespace(true);
	    // factory.setCoalescing(true);
	    // factory.setIgnoringComments(true);
	    builder = factory.newDocumentBuilder();
	    tFactory = TransformerFactory.newInstance();
	    transformer = tFactory.newTransformer();
	    dir = ProviderDescriptorFactory.getInstance();
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }

    /**
     * Verify the given XML Signature.  The Element must be a ds:Signature.
     * Uses the key from the Liberty provider with the given ID.
     * @param sigElement the signature element
     * @param provider the provider ID
     * @return true if signature is valid and is from the given provider,
     * otherwise false.
     */
    public static boolean verify(Element sigElement, String provider) {
        DebugUtils.debugTime("XMLUtils", "Entering verify",
                             DebugUtils.LOW_LEVEL_FUNC);
	if (Config.checkSignatures == false) {
	    System.err.println("WARNING: Signature checking disabled!");
	    return true;
	}
	if (Config.debug) {
	    System.err.println("verify sigElement:" + sigElement
			       + " provider:" + provider);
	}
	try {
	    ProviderDescriptor prov = dir.find(provider);
	    PublicKey key = prov.getKeyInfo();
	    if (Config.debug) {
		System.err.println("key:" + key + " prov:" + prov);
	    }
	    if (Config.isolatedSignatureVerification) {
	        Document tempDoc = builder.newDocument();
	        tempDoc.appendChild(tempDoc.importNode(sigElement.getParentNode(), true));
	        NodeList list = tempDoc.getElementsByTagNameNS(XMLElement.DSIG_NS_URI, "Signature");
		if (list.getLength() == 0) {
		    return false;
		}
		sigElement = (Element) list.item(0);
	    }
	    XMLSignature sig = new XMLSignature(sigElement, "");
            DebugUtils.debugTime("XMLUtils", "verifying",
                                 DebugUtils.HIGH_LEVEL_FUNC);
	    boolean rv = sig.checkSignatureValue(key);
            DebugUtils.debugTime("XMLUtils", "done verifying",
                                 DebugUtils.HIGH_LEVEL_FUNC);
	    if (Config.debug) {
		System.err.println("rv:" + rv);
		System.err.println("post:" + new String(
		    sig.getSignedInfo()
		       .getReferencedContentAfterTransformsItem(0).getBytes()));
		System.err.println("csi:"
		        + new String(sig.getSignedInfo()
					.getCanonicalizedOctetStream()));
	    }
            DebugUtils.debugTime("XMLUtils", "Leaving verify",
                                 DebugUtils.LOW_LEVEL_FUNC);
	    return rv;
	} catch (Exception e) {
	    e.printStackTrace();
	    return false;
	}
    }

    /**
     * Signs the given element.  Appends the ds:Signature element as
     * a child.  Uses the configured key.
     * @param e the Element to be signed
     */
    public static void sign(Element e) {
        DebugUtils.debugTime("XMLUtils", "Entering sign",
                             DebugUtils.LOW_LEVEL_FUNC);
	try {
	    KeyStore ks = KeyStore.getInstance(KeyStore.getDefaultType());
	    if (Config.xmlDebug) {
		System.err.println("keystore:" + Config.signingKeystore);
		System.err.println("pass:" + Config.signingKeystorePass);
		System.err.println("alias:" + Config.signingKeyAlias);
		System.err.println("pass:" + Config.signingKeyPass);
            }
	    ks.load(new java.io.FileInputStream(Config.signingKeystore),
	     Config.signingKeystorePass.toCharArray());
	    PrivateKey signingKey;
	    signingKey = (PrivateKey) ks.getKey(Config.signingKeyAlias,
					 Config.signingKeyPass.toCharArray());

	    nsFixup(e, new HashMap(), true);

	    // We have to do the signing in another document.  Oh, well.
	    Document tempDoc = builder.newDocument();
	    tempDoc.appendChild(tempDoc.importNode(e, true));
	    XMLSignature sig;
	    sig = new XMLSignature(tempDoc, "", Config.signingKeyAlg);
	    Element sigNode = sig.getElement();
	    tempDoc.getDocumentElement().appendChild(sigNode);
	    Transforms t = new Transforms(tempDoc);
	    XPathContainer xc = new XPathContainer(tempDoc);
	    xc.setXPath("ancestor-or-self::node()/ds:Signature/ds:SignedInfo/ds:Reference/ds:Transforms/ds:Transform/ds:XPath = here()");
	    t.addTransform(Transforms.TRANSFORM_XPATH, xc.getElement());
	    t.addTransform(Transforms.TRANSFORM_ENVELOPED_SIGNATURE);
	    t.addTransform(Transforms.TRANSFORM_C14N_EXCL_OMIT_COMMENTS);
	    sig.addDocument("", t,
			    Constants.ALGO_ID_DIGEST_SHA1);
	    long d = 0;
	    if (Config.debug) {
		System.err.println("(signing)");
		d = new java.util.Date().getTime();
	    }
            DebugUtils.debugTime("XMLUtils", "signing",
                                 DebugUtils.HIGH_LEVEL_FUNC);
	    sig.sign(signingKey);
            DebugUtils.debugTime("XMLUtils", "done signing",
                                 DebugUtils.HIGH_LEVEL_FUNC);
	    if (Config.debug) {
		d = new java.util.Date().getTime() - d;
		System.err.println("(done signing -- " + d + " ms)");

		System.err.println("post:" + new String(
		    sig.getSignedInfo()
		       .getReferencedContentAfterTransformsItem(0).getBytes()));
		System.err.println("csi:"
		        + new String(sig.getSignedInfo()
					.getCanonicalizedOctetStream()));
	    }

	    e.appendChild(e.getOwnerDocument().importNode(sigNode, true));
            DebugUtils.debugTime("XMLUtils", "Leaving sign",
                                 DebugUtils.LOW_LEVEL_FUNC);
	} catch (Exception ex) {
	    ex.printStackTrace();
	}
    }

    /**
     * Traverses the document, adding any needed xmlns:* attributes
     * for namespaces that are used.
     */
    public static void nsFixup(Document doc) {
	Map map = new HashMap();
	Element e = doc.getDocumentElement();
	nsFixup(e, map, true);
    }

    private static void nsFixup(Element e, Map map, boolean setAttrs) {
	if (Config.xmlDebug) {
	    System.err.println("nsFixup e:" + e + " map:" + map
		               + " setAttrs:" + setAttrs);
	}
	NamedNodeMap attrs = e.getAttributes();
	for (int i = 0; i < attrs.getLength(); i++) {
	    Attr a = (Attr) attrs.item(i);
	    if (a.getName().indexOf("xmlns:") == 0) {
		if (Config.xmlDebug) { System.err.println("(not traversing)"); }
		return;
	    }
	}
	String prefix = e.getPrefix();
	String namespaceURI = e.getNamespaceURI();
	if (prefix == null && namespaceURI != null) {
	    prefix = "";
	}
	if (prefix != null) {
	    String currentNS = (String) map.get(prefix);
	    if (currentNS == null) {
		map.put(prefix, namespaceURI);
	    } else {
		if (!namespaceURI.equals(currentNS)) {
		    if (Config.xmlDebug) {
			System.err.println("prefix:" + prefix
					   + " currentNS:" + currentNS
					   + " namespaceURI:" + namespaceURI
					   + " element:" + e);
		    }
		    /*
		     * Use new map when recursing -- but this really shouldn't
		     * happen.  Note that all declarations will be repeated
		     * here, not just those which need redefinition at this
		     * level.
		     */
		    map = new HashMap();
		    setAttrs = true;
		    map.put(prefix, namespaceURI);
		}
		/*
		 * else, current map is already correct and no change is
		 * necessary
		 */
	    }
	}
	for (int i = 0; i < attrs.getLength(); i++) {
	    Attr a = (Attr) attrs.item(i);
	    String attrPrefix = a.getPrefix();
	    String attrNS = a.getNamespaceURI();
	    if (attrPrefix != null && attrNS != null) {
		String oldNS = (String) map.get(attrPrefix);
		if (oldNS == null || !(oldNS.equals(attrNS))) {
		    map.put(attrPrefix, attrNS);
		}
	    }
	}
	NodeList list = e.getChildNodes();
	for (int i = 0; i < list.getLength(); i++) {
	    Node n = list.item(i);
	    if (!(n instanceof Element)) { continue; }
	    nsFixup((Element) n, map, false);
	}
	if (setAttrs) {
	    Iterator i = map.entrySet().iterator();
	    while (i.hasNext()) {
		Map.Entry entry = (Map.Entry) i.next();
		String nsprefix = (String) entry.getKey();
		String namespace = (String) entry.getValue();
		if (!nsprefix.equals("")) {
		    nsprefix = "xmlns:" + nsprefix;
		}
		if (Config.xmlDebug) {
		    System.err.println(e + " setAttribute nsprefix:" + nsprefix
					 + " namespace:" + namespace);
		}
		e.setAttributeNS(XMLElement.XMLNS_NS_URI, nsprefix, namespace);
	    }
	}
    }
	

    /**
     * Parses an XML file.
     * @param filename the name of the file
     * @return the parsed Document
     */
    public static Document parseFile(String filename) throws Exception {
	return builder.parse(new FileInputStream(filename));
    }

    /**
     * Parses a string containing XML.
     * @param xml the string.
     * @return the parsed Document
     */
    public static Document parse(String xml) throws Exception {
	DocumentBuilder myBuilder = factory.newDocumentBuilder();
	return myBuilder.parse(new ByteArrayInputStream(xml.getBytes()));
    }

    /**
     * Creates a new empty document with namespace support.
     * @return the new Document
     */
    public static Document newDoc() {
	return builder.newDocument();
    }

    /**
     * Transforms an XMLElement into a string with XML in it.
     * @param elt the XMLElement.
     * @return the string.
     */
    public static String serialize(XMLElement elt) throws Exception {
	Document doc = builder.newDocument();
	doc.appendChild(elt.getElement(doc));
	nsFixup(doc);
	ByteArrayOutputStream os = new ByteArrayOutputStream();
	transformer.transform(new DOMSource(doc), new StreamResult(os));
	return os.toString();
    }

    /**
     * Really dumb comparison function.  Probably better to not use this.
     * @param lhs left hand side
     * @param rhs right hand side
     * @return -1, 0, or 1, if left hand side is greater, equal, or lesser
     */
    public static int compare(XMLElement lhs, XMLElement rhs) throws ClassCastException {
	try {
		String lhsdoc = lhs.toString();
		String rhsdoc = rhs.toString();
		return lhsdoc.compareTo(rhsdoc);
	} catch (Exception e) {
		throw new ClassCastException();
	}
    }

    /**
     * Matches an Element against a tag name and a namespace identifier.
     * @param e the element.
     * @param ns the namespace URI.
     * @param name the tag name.
     * @return true if the element's tag name equals e and its namespace
     * is either null or equals ns.
     */
    public static boolean isElement(Element e, String ns, String name) {
        String ens = e.getNamespaceURI();
        String ename = e.getLocalName();
        if (Config.xmlDebug) { System.err.println("match: " + ns + ", " + name); }
        if (Config.xmlDebug) { System.err.println("getNamespaceURI(): " + ens); }
        if (Config.xmlDebug) { System.err.println("getLocalName(): " + ename); }
        if (Config.xmlDebug) { System.err.println("getTagName(): " + e.getTagName()); }
        if (ename == null || ens == null) {
            return e.getTagName().equals(name);
        } else {
            return (ens.equals(ns) && ename.equals(name));
        }
    }    
}
