/*
 * $Id: SAMLAuthorityBinding.java,v 1.5.2.1 2002/09/17 01:52:30 jsergent Exp $
 * SAMLAuthorityBinding.java
 *
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.experimentalstuff.com/Technologies/IPL/download.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2002 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 */

package com.sun.liberty.saml;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * Data bindings for saml:AuthorityBinding element.  Get and set methods
 * correspond to child elements and attributes in the XML schema for this
 * element.
 *
 * @author Jonathan Sergent
 * @version $Revision: 1.5.2.1 $
 */

public class SAMLAuthorityBinding extends SAMLElement {

    public final static String AUTHORITY_KIND_SUBJECT_QUERY
    = SAMLP_NS_ID + ":SubjectQuery";
    public final static String AUTHORITY_KIND_AUTHENTICATION_QUERY
	= SAMLP_NS_ID + ":AuthenticationQuery";
    public final static String AUTHORITY_KIND_ATTRIBUTE_QUERY
	= SAMLP_NS_ID + ":AttributeQuery";
    public final static String AUTHORITY_KIND_AUTHORIZATION_DECISION_QUERY
	= SAMLP_NS_ID + ":AuthorizationDecisionQuery";

    private String authorityKind = null;
    private String location = null;
    private String binding = null;


    public void parse(Element e) {
	setAuthorityKind(e.getAttributeNS(null, "AuthorityKind"));
	setLocation(e.getAttributeNS(null, "Location"));
	setBinding(e.getAttributeNS(null, "Binding"));
    }
    
    public Element getElement(Document doc) {
	Element s = doc.createElementNS(SAML_NS_URI, "saml:AuthorityBinding");
	addAttributesTo(s);
	return s;
    }

    protected void addAttributesTo(Element s) {
	if (authorityKind != null) {
	    s.setAttributeNS(null, "AuthorityKind", authorityKind);
	}
	if (location != null) {
	    s.setAttributeNS(null, "Location", location);
	}
	if (binding != null) {
	    s.setAttributeNS(null, "Binding", binding);
	}
    }

    public String getAuthorityKind() {
	return authorityKind;
    }

    public void setAuthorityKind(String value) {
	authorityKind = value;
    }

    public String getLocation() {
	return location;
    }

    public void setLocation(String value) {
	location = value;
    }

    public String getBinding() {
	return binding;
    }

    public void setBinding(String value) {
	binding = value;
    }
}
