/*
 * $Id: SAMLAuthorizationDecisionStatement.java,v 1.5.2.1 2002/09/17 01:52:30 jsergent Exp $
 * SAMLAuthorizationDecisionStatement.java
 *
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.experimentalstuff.com/Technologies/IPL/download.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2002 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 */

package com.sun.liberty.saml;
import java.util.List;
import java.util.LinkedList;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * Data bindings for saml:AuthorizationDecisionStatement element.  Get and
 * set methods correspond to child elements and attributes in the XML
 * schema for this element.
 *
 * @author Jonathan Sergent
 * @version $Revision: 1.5.2.1 $
 */

public class SAMLAuthorizationDecisionStatement extends SAMLSubjectStatement {
    String resource = "";
    SAMLDecisionType decision = SAMLDecisionType.INDETERMINATE;
    List actions = null;
    List evidence = null;

    public void parse(Element e) {
	super.parse(e);
	setResource(e.getAttributeNS(null, "Resource"));
	setDecision(SAMLDecisionType.valueOf(e.getAttributeNS(null, "Decision")));
	NodeList list = e.getChildNodes();
	for (int j = 0; j < list.getLength(); j++) {
	    if (list.item(j) instanceof org.w3c.dom.Text) { continue; }
	    Element child = (Element) list.item(j);
	    if (isElement(child, SAML_NS_URI, "Action")) {
		SAMLAction c = new SAMLAction();
		c.parse(child);
		List set = getActions();
		if (set == null) {
		    set = new LinkedList();
		}
		set.add(c);
		setActions(set);
	    } else if (isElement(child, SAML_NS_URI, "Evidence")) {
		SAMLEvidence c = new SAMLEvidence();
		c.parse(child);
		List set = getEvidence();
		if (set == null) {
		    set = new LinkedList();
		}
		set.add(c);
		setEvidence(set);		
	    }
	}
    }

    public Element getElement(Document doc) {
	Element s;
	s = doc.createElementNS(SAML_NS_URI,
			 SAML_NS_ID + ":AuthorizationDecisionStatement");
	addAttributesTo(s);
	addChildrenTo(s);
	return s;
    }

    protected void addChildrenTo(Element s) {
	super.addChildrenTo(s);
	iterateAddChildren(s, actions);
	iterateAddChildren(s, evidence);
    }

    protected void addAttributesTo(Element s) {
	s.setAttributeNS(null, "Resource", resource);
	s.setAttributeNS(null, "Decision", decision.toString());
    }

    public List getActions() {
	return actions;
    }

    public void setActions(List value) {
	actions = value;
    }

    public SAMLDecisionType getDecision() {
	return decision;
    }

    public void setDecision(SAMLDecisionType value) {
	decision = value;
    }

    public List getEvidence() {
	return evidence;
    }

    public void setEvidence(List value) {
	evidence = value;
    }

    public String getResource() {
	return resource;
    }

    public void setResource(String value) {
	resource = value;
    }
}
