/*
 * $Id: SAMLElement.java,v 1.6.2.1 2002/09/17 01:52:31 jsergent Exp $
 * SAMLElement.java
 *
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.experimentalstuff.com/Technologies/IPL/download.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2002 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 */

package com.sun.liberty.saml;

import com.sun.liberty.XMLElement;
import java.io.ByteArrayOutputStream;
import java.io.OutputStream;
import java.security.SecureRandom;
import javax.mail.internet.MimeUtility;
import java.text.SimpleDateFormat;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * Base class for data bindings of SAML elements.
 *
 * @author Jonathan Sergent
 * @version $Revision: 1.6.2.1 $
 */

public abstract class SAMLElement extends XMLElement {
    public static final String URI_PREFIX = "urn:oasis:names:tc:SAML:1.0:";

    /* XML namespace identifiers for SAML assertion schema */
    public static final String SAML_NS_ID = "saml";
    public static final String SAML_NS_URI = URI_PREFIX + "assertion";

    public static final String SAMLP_NS_ID = "samlp";
    public static final String SAMLP_NS_URI = URI_PREFIX + "protocol";

    /* SAML binding identifiers */
    public static final String SOAP_BINDING
	= URI_PREFIX + "bindings:SOAP-binding";

    /* Auth methods */
    public static final String AM_PASSWORD = URI_PREFIX + "am:password";
    public static final String AM_KERBEROS = "urn:ietf:rfc:1510";
    public static final String AM_SRP = "urn:ietf:rfc:2945";
    public static final String AM_HW_TOKEN = URI_PREFIX + "am:HardwareToken";
    public static final String AM_SSL_TLS = "urn:ietf:rfc:2246";
    public static final String AM_X509 = URI_PREFIX + "am:X509-PKI";
    public static final String AM_PGP = URI_PREFIX + "am:PGP";
    public static final String AM_SPKI = URI_PREFIX + "am:SPKI";
    public static final String AM_XKMS = URI_PREFIX + "am:XKMS";
    public static final String AM_XMLDSIG = "urn:ietf:rfc:3075";

    /* Action namespaces and values to be used with them */
    public static final String ACTIONNS_RWEDC = URI_PREFIX + "action:rwedc";
    public static final String RWEDC_READ = "Read";
    public static final String RWEDC_WRITE = "Write";
    public static final String RWEDC_EXECUTE = "Execute";
    public static final String RWEDC_DELETE = "Delete";
    public static final String RWEDC_CONTROL = "Control";
    public static final String ACTIONNS_RWEDC_NEG
	= URI_PREFIX + "action:rwedc-negation";
    public static final String RWEDC_NEG_READ = "~" + RWEDC_READ;
    public static final String RWEDC_NEG_WRITE = "~" + RWEDC_WRITE;
    public static final String RWEDC_NEG_EXECUTE = "~" + RWEDC_EXECUTE;
    public static final String RWEDC_NEG_DELETE = "~" + RWEDC_DELETE;
    public static final String RWEDC_NEG_CONTROL = "~" + RWEDC_CONTROL;
    public static final String ACTIONNS_GHPP = URI_PREFIX + "action:ghpp";
    public static final String GHPP_GET = "GET";
    public static final String GHPP_HEAD = "HEAD";
    public static final String GHPP_PUT = "PUT";
    public static final String GHPP_POST = "POST";
    public static final String ACTIONNS_UNIX = URI_PREFIX + "action:unix";
    public static final int UNIX_ISUID = 0x800;
    public static final int UNIX_ISGID = 0x400;
    public static final int UNIX_ISVTX = 0x200;
    public static final int UNIX_IRWXU = 00700;
    public static final int UNIX_IRUSR = 00400;
    public static final int UNIX_IWUSR = 00200;
    public static final int UNIX_IXUSR = 00100;
    public static final int UNIX_IRWXG = 00070;
    public static final int UNIX_IRGRP = 00040;
    public static final int UNIX_IWGRP = 00020;
    public static final int UNIX_IXGRP = 00010;
    public static final int UNIX_IRWXO = 00007;
    public static final int UNIX_IROTH = 00004;
    public static final int UNIX_IWOTH = 00002;
    public static final int UNIX_IXOTH = 00001;

    public static final String CM_HOLDER_OF_KEY
	= URI_PREFIX + "cm:holder-of-key";
    public static final String CM_SENDER_VOUCHES
	= URI_PREFIX + "cm:sender-vouches";
    public static final String CM_ARTIFACT = URI_PREFIX + "cm:artifact-01";
    public static final String CM_BEARER = URI_PREFIX + "cm:bearer";

    public static final String SC_SUCCESS = "samlp:Success";
    public static final String SC_VERSION_MISMATCH = "samlp:VersionMismatch";
    public static final String SC_REQUESTER = "samlp:Requester";
    public static final String SC_RESPONDER = "samlp:Responder";

    /**
     * Generates a new SAML IDType value using SecureRandom and a base64
     * encoder.  Return value contains 168 bits of randomness.
     * @return a String of base64-encoded secure random data.
     */
    public static String generateIDType() {
        byte[] bytes = new byte[21];
        secureRandom.nextBytes(bytes);
        try {
            ByteArrayOutputStream encoded = new ByteArrayOutputStream();
            OutputStream encoder = MimeUtility.encode(encoded, "base64");
            encoder.write(bytes);
            encoder.close();
            return encoded.toString();
        } catch (Exception e) {
            /* Shouldn't happen */
            e.printStackTrace();
            return null;
        }
    }
}
