/*
 * $Id: SAMLNameIdentifier.java,v 1.7.2.1 2002/09/17 01:52:31 jsergent Exp $
 * SAMLCondition.java
 *
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.experimentalstuff.com/Technologies/IPL/download.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2002 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 */

package com.sun.liberty.saml;
import java.util.StringTokenizer;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * Data bindings for saml:NameIdentifier element.  Also contains utility
 * functions for "flat string" representation and storage of the name
 * identifiers.  Get and set methods correspond to child elements and
 * attributes in the XML schema for this element.
 *
 * @author Jonathan Sergent
 * @version $Revision: 1.7.2.1 $
 */ 

public class SAMLNameIdentifier extends SAMLElement {
    private String value = null;
    private String nameQualifier = null;
    private String format = null;


    public void parse(Element e) {
	setNameQualifier(e.getAttributeNS(null, "NameQualifier"));
	setFormat(e.getAttributeNS(null, "Format"));
	setValue(e.getFirstChild().getNodeValue());
    }

    public void parse(String str) {
	int j = str.indexOf("|");
	int k = str.indexOf("|", j + 1);
	if (j == -1) { return; }
	setValue(str.substring(0, j));
	if (k == -1) { return; }
	setNameQualifier(str.substring(j + 1, k));
	setFormat(str.substring(k + 1));
    }

    public String toString() {
	return (value == null ? "" : value) + "|"
		+ (nameQualifier == null ? "" : nameQualifier) + "|"
		+ (format == null ? "" : format);
    }

    public Element getElement(Document doc) {
        Element n = doc.createElementNS(SAML_NS_URI, "saml:NameIdentifier");
        addAttributesTo(n);
        addChildrenTo(n);
        return n;
    }

    protected void addAttributesTo(Element n) {
        if (nameQualifier != null && !nameQualifier.equals("")) {
            n.setAttributeNS(null, "NameQualifier", nameQualifier);
        }
        if (format != null && !format.equals("")) {
            n.setAttributeNS(null, "Format", format);
        }
    }

    protected void addChildrenTo(Element n) {
        n.appendChild(n.getOwnerDocument().createTextNode(value));
    }
    
        
    public String getValue() {
        return value;
    }

    public void setValue(String name) {
        value = name;
    }

    public String getNameQualifier() {
        return nameQualifier;
    }

    public void setNameQualifier(String value) {
        nameQualifier = value;
    }

    public String getFormat() {
        return format;
    }

    public void setFormat(String value) {
        format = value;
    }
}
