/*
 * $Id: SAMLSubject.java,v 1.7.2.1 2002/09/17 01:52:32 jsergent Exp $
 * SAMLSubject.java
 *
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.experimentalstuff.com/Technologies/IPL/download.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2002 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 */

package com.sun.liberty.saml;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * Data bindings for saml:Subject element.  Get and set methods correspond
 * to child elements and attributes in the XML schema for this element.
 * @author Jonathan Sergent 
 * @version $Revision: 1.7.2.1 $ 
 */ 

public class SAMLSubject extends SAMLElement {
    private SAMLNameIdentifier nameIdentifier = null;
    private SAMLSubjectConfirmation subjectConfirmation = null;

    
    public void parse(Element e) {
	NodeList list = e.getChildNodes();
	for (int j = 0; j < list.getLength(); j++) {
	    if (list.item(j) instanceof org.w3c.dom.Text) { continue; }
	    Element child = (Element) list.item(j);
	    if (isElement(child, SAML_NS_URI, "NameIdentifier")) {
		SAMLNameIdentifier c = new SAMLNameIdentifier();
		c.parse(child);
		setNameIdentifier(c);
	    } else if (isElement(child, SAML_NS_URI, "SubjectConfirmation")) {
		SAMLSubjectConfirmation c = new SAMLSubjectConfirmation();
		c.parse(child);
		setSubjectConfirmation(c);
	    }
	}
    }
    
    public Element getElement(Document doc) {
        Element s = doc.createElementNS(SAML_NS_URI, "saml:Subject");
        addChildrenTo(s);
        return s;
    }

    protected void addChildrenTo(Element s) {
        if (nameIdentifier != null) {
            s.appendChild(nameIdentifier.getElement(s.getOwnerDocument()));
        }
        if (subjectConfirmation != null) {
            s.appendChild(subjectConfirmation.getElement(s.getOwnerDocument()));
        }
    }

    public SAMLNameIdentifier getNameIdentifier() {
	return nameIdentifier;
    }

    public void setNameIdentifier(SAMLNameIdentifier value) {
	nameIdentifier = value;
    }

    public SAMLSubjectConfirmation getSubjectConfirmation() {
	return subjectConfirmation;
    }

    public void setSubjectConfirmation(SAMLSubjectConfirmation value) {
	subjectConfirmation = value;
    }
}
