/*
 * $Id: SAMLSubjectConfirmation.java,v 1.7.2.1 2002/09/17 01:52:32 jsergent Exp $
 * SAMLSubjectConfirmation.java
 *
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.experimentalstuff.com/Technologies/IPL/download.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2002 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 */

package com.sun.liberty.saml;
import java.util.Iterator;
import java.util.List;
import java.util.LinkedList;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * Data bindings for saml:SubjectConfirmation element.  Get and set methods
 * correspond to child elements and attributes in the XML schema for this
 * element.
 * @author Jonathan Sergent 
 * @version $Revision: 1.7.2.1 $ 
 */ 

public class SAMLSubjectConfirmation extends SAMLElement {
    List confirmationMethods = null;
    String subjectConfirmationData = null;
    Element keyInfo = null;
    

    public void parse(Element e) {
	NodeList list = e.getChildNodes();
	for (int j = 0; j < list.getLength(); j++) {
	    if (list.item(j) instanceof org.w3c.dom.Text) { continue; }
	    Element child = (Element) list.item(j);
	    if (isElement(child, SAML_NS_URI, "ConfirmationMethod")) {
		List set = getConfirmationMethods();
		if (set == null) {
		    set = new LinkedList();
		}
		set.add(child.getFirstChild().getNodeValue());
		setConfirmationMethods(set);
	    } else if (isElement(child, SAML_NS_URI, "SubjectConfirmationData")) {
		setSubjectConfirmationData(child.getFirstChild().getNodeValue());
	    } else if (isElement(child, DSIG_NS_URI, "KeyInfo")) {
		setKeyInfo(child);
	    }
	}
    }
    
    public Element getElement(Document doc) {
        Element sc = doc.createElementNS(SAML_NS_URI, "saml:SubjectConfirmation");
        addChildrenTo(sc);
        return sc;
    }

    protected void addChildrenTo(Element sc) {
        Document doc = sc.getOwnerDocument();
        if (confirmationMethods != null) {
            Iterator i = confirmationMethods.iterator();
            while (i.hasNext()) {
                Element m = doc.createElementNS(SAML_NS_URI, "saml:ConfirmationMethod");
                sc.appendChild(m);
                m.appendChild(doc.createTextNode((String) i.next()));
            }
        }
        
        if (subjectConfirmationData != null) {
            Element scd = doc.createElementNS(SAML_NS_URI, "saml:SubjectConfirmationData");
            sc.appendChild(scd);
            scd.appendChild(doc.createTextNode(subjectConfirmationData));
        }
        
	if (keyInfo != null) {
	    sc.appendChild(doc.importNode(keyInfo, true));
	}
    }


    public List getConfirmationMethods() {
        return confirmationMethods;
    }

    public void setConfirmationMethods(List value) {
        confirmationMethods = value;
    }

    public String getSubjectConfirmationData() {
        return subjectConfirmationData;
    }

    public void setSubjectConfirmationData(String value) {
        subjectConfirmationData = value;
    }

    public Element getKeyInfo() {
        return keyInfo;
    }

    public void setKeyInfo(Element value) {
        keyInfo = value;
    }
}
