package com.sun.liberty.servlets;

import ucb.eecs.sahara.sso.DebugUtils;
import ucb.eecs.sahara.sso.sp.AuthnControlSession;
import ucb.eecs.sahara.sso.authn.FirewallController;
import ucb.eecs.sahara.sso.authn.LibertyAuthenticator;
import ucb.eecs.sahara.sso.authn.AuthenticatorResult;
import ucb.eecs.sahara.sso.authn.LibertyAssertionControl;

import com.sun.liberty.Config;
import com.sun.liberty.LibertyAssertion;
import com.sun.liberty.LibertyHTMLPostAuthnResponse;
import com.sun.liberty.LibertyUserSession;
import java.io.InputStream;
import java.util.List;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import javax.servlet.http.Cookie;
import javax.servlet.RequestDispatcher;
import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;

/**
 * @author Ana Sanz Merino
 * @version $Revision: 1.1.1.1 $
 */

public class AssertionServlet extends HttpServlet {

    /**
     * Utility function to locate user's LibertyUserSession object from the
     * HttpSession.  If none is found, a new one is created.
     */
    private static LibertyUserSession getLibertySession(HttpServletRequest req) {
        HttpSession ses = req.getSession(true);
        LibertyUserSession lus;
        lus = (LibertyUserSession) ses.getAttribute("liberty.session");
        if (lus == null) {
            lus = new LibertyUserSession();
            ses.setAttribute("liberty.session", lus);
        }
	return lus;
    }    

    private static AuthnControlSession getControlSession(HttpServletRequest req) {
        HttpSession ses = req.getSession(true);
        AuthnControlSession ctls;
        ctls = (AuthnControlSession) ses.getAttribute("control.session");
        if (ctls == null) {
            ctls = new AuthnControlSession();
            ses.setAttribute("control.session", ctls);
        }
        return ctls;
    }

    private void handleAuthenticationResult(HttpServletRequest request,
                                            HttpServletResponse response,
                                            AuthenticatorResult authn,
                                            LibertyAssertionControl aCtl)
                 throws ServletException
    {
        DebugUtils.debugTime("AssertionServlet", "Entering handleAuthenticationResult",
                             DebugUtils.LOW_LEVEL_FUNC);

        LibertyUserSession lus = getLibertySession(request);
        String idpid = lus.getPendingIDP();
        String relay = lus.getPendingRelayURL();
        if (Config.debug) {
            System.err.println("pending relay:" + relay);
            System.err.println("pending idpid:" + idpid);
        }
        /*
         * We do not null out the pending relay URL.  This way if a request
         * is mistakenly resubmitted (i.e. back or reload button), the user
         * ends up in the right place, but the request is not processed
         * again.  It is not clear if this is necessary, but it doesn't hurt.
         */

	    try {
     	    if (authn.equals(AuthenticatorResult.AUTHN_SUCCESS) &&
                !aCtl.federate) {
                /* Remove the door open in the firewall for remote authentication */
                if (Config.debug) {
                    System.err.println("handle: removing authentication door from filter");
                }
                FirewallController.removeURLAccess(request, aCtl.redirectedServer);

                /* If requested, allow user to access the external network */
                AuthnControlSession ctls = getControlSession(request);
                if (ctls.getExternalAccess() == true) {
                    if (Config.debug) {
                        System.err.println("handle: adding remote IP to the filter");
                    }

                    /* Wait some milliseconds to avoid race conditions
                     *  If the firewall is immediately accessed after the 
                     *  previous access, adding the new address fails some
                     *  times
                     */
                    java.lang.Thread.sleep(20);
                    FirewallController.giveAccess(request);
                }

                /* Add cookie */
                DebugUtils.debugTime("AssertionServlet", "Adding cookie",
                                     DebugUtils.LOW_LEVEL_FUNC);

                Cookie cookie = new Cookie("idp", idpid);
                cookie.setDomain(".cs.berkeley.edu");
                if (Config.debug) {
                    System.err.println("created cookie");
                    System.err.println("cookie value: " + cookie.getValue());
                    System.err.println("cookie domain: " + cookie.getDomain());
                }
                response.addCookie(cookie);
                if (Config.debug) {System.err.println("added cookie");}
            }

            /* I could close the hole in firewall if no federate, AUTHN_FAILED,
             *  but I know the redirectURL. This means that, although authenticating
             *  the assertion failed, it was a valid assertion
             */

            String path = null;
	        if (authn.equals(AuthenticatorResult.AUTHN_FAILED)) {
                path = Config.failedAuthnPage;
            } else if (relay != null) {
	            if (Config.debug) { System.err.println("Redirecting to: " + relay); }
	            response.sendRedirect(relay);
                    DebugUtils.debugTime("AssertionServlet", "Redirecting",
                                         DebugUtils.LOW_LEVEL_FUNC);
                return;
            } else {
                path = Config.successfulLibertyAuthnPage;
            }

            ServletConfig cfg = getServletConfig();
            ServletContext ctx = cfg.getServletContext();
            RequestDispatcher disp;
            disp = ctx.getRequestDispatcher(path);
            disp.forward(request, response);
            DebugUtils.debugTime("AssertionServlet", "Forwarding",
                                 DebugUtils.LOW_LEVEL_FUNC);
        } catch (Exception e) {
            throw new ServletException(e);
        }
    }

    /**
     * Implements standard servlet doGet interface.  Handles incoming
     * artifact redirections.  Dereferences artifact then calls handleAssertion
     * with the result.
     */
    public void doGet(HttpServletRequest req,
                      HttpServletResponse resp)
                throws ServletException
    {
        DebugUtils.debugTime("AssertionServlet", "Entering doGet",
                             DebugUtils.HIGH_LEVEL_FUNC);

    	// GET means artifact dereference
	    String art = req.getParameter("SAMLart");
	    if (art == null) {
	        throw new ServletException("Artifact is not present");
	    }

        try {
            AuthenticatorResult authn = null;
            LibertyAssertionControl aCtl = new LibertyAssertionControl();
            authn = LibertyAuthenticator.authenticateArtifact(req, art, aCtl);
            handleAuthenticationResult(req, resp, authn, aCtl);
	} catch (Exception e) {
	    throw new ServletException(e);
	}

        DebugUtils.debugTime("AssertionServlet", "Leaving doGet",
                             DebugUtils.HIGH_LEVEL_FUNC);
    }


    /**
     * Implements standard servlet doPost interface.  Handles incoming
     * HTML form post assertions.  Decodes assertion, then calls
     * handleAssertion with the result.
     */
    public void doPost(HttpServletRequest req,
                       HttpServletResponse resp)
                throws ServletException
    {
        DebugUtils.debugTime("AssertionServlet", "Entering doPost",
                             DebugUtils.HIGH_LEVEL_FUNC);

        AuthenticatorResult authn = null;
        LibertyAssertionControl aCtl = new LibertyAssertionControl();

     	LibertyHTMLPostAuthnResponse ar = new LibertyHTMLPostAuthnResponse();
    	ar.setReceivedFrom(getLibertySession(req).getPendingIDP());
	    try {
	        InputStream r = req.getInputStream();
	        int l = req.getContentLength();
	        int o = 0;
	        byte[] c = new byte[l];
	        while (o < l) {
		        o += r.read(c, o, l-o);
	        }
	        String s = new String(c);
	        if (Config.debug) { System.err.println("ar parse s:" + s); }
	        ar.parse(s);

			if (Config.debug) {
				/*
				 * Note that we don't actually care if the response is signed,
				 * only the assertion inside it.  So this is just informational.
				 */
				System.err.println("pendingIDP:"
					  + getLibertySession(req).getPendingIDP());
				System.err.println("ar signedBy:" + ar.getSignedBy());
			}
			if (ar.getRecipient() != null && !ar.getRecipient().equals("")
				&& !ar.getRecipient().equals(Config.localProviderID)) {
				/* Assertion isn't for us! */
				if (Config.debug) {
					System.err.println("Recipient mismatch");
					System.err.println("recipient:" + ar.getRecipient());
				    System.err.println("id:" + Config.localProviderID);
				}
				authn = LibertyAuthenticator.authenticateAssertion(req, null, aCtl);
			} else {
				LibertyAssertion a = null;
	            List assertions = ar.getAssertions();
	            if (assertions != null) {
	                a = (LibertyAssertion) assertions.iterator().next();
	            }   
                authn = LibertyAuthenticator.authenticateAssertion(req, a, aCtl);
            }

            handleAuthenticationResult(req, resp, authn, aCtl);
	    } catch (Exception e) {
	        throw new ServletException(e);
	    }

        DebugUtils.debugTime("AssertionServlet", "Leaving doPost",
                             DebugUtils.HIGH_LEVEL_FUNC);
    }

}
