/*
 * $Id: FedTermServlet.java,v 1.4.2.1 2002/09/17 01:52:33 jsergent Exp $
 * FedTermRequest.java
 *
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.experimentalstuff.com/Technologies/IPL/download.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2002 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 */

package com.sun.liberty.servlets;
import com.sun.liberty.Config;
import com.sun.liberty.ProviderDescriptorFactory;
import com.sun.liberty.ProviderDescriptor;
import com.sun.liberty.LibertyUserSession;
import com.sun.liberty.LibertyURLFederationTerminationNotification;
import com.sun.liberty.RequestProcessorAction;
import com.sun.liberty.LibertyFederationTerminationNotification;
import com.sun.liberty.FederationTerminationNotificationProcessor;
import java.net.URL;
import javax.servlet.ServletException;
import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

/**
 * Servlet to handle incoming front channel FederationTerminationNotifications
 *
 * @author Jonathan Sergent
 * @version $Revision: 1.4.2.1 $
 */

public class FedTermServlet extends HttpServlet {
    static ProviderDescriptorFactory dir;

    static {
	try {
	    dir = ProviderDescriptorFactory.getInstance();
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }

    /**
     * Utility function to locate user's LibertyUserSession object from the
     * HttpSession.  If none is found, a new one is created.
     */
    private static LibertyUserSession getLibertySession(HttpServletRequest req) {
        HttpSession ses = req.getSession(true);
        LibertyUserSession lus;
        lus = (LibertyUserSession) ses.getAttribute("liberty.session");
        if (lus == null) {
            lus = new LibertyUserSession();
            ses.setAttribute("liberty.session", lus);
        }
	return lus;
    }

    /**
     * Implements standard servlet doGet interface.  Handles incoming
     * FederationTerminationNotifications that are URL-encoded.  SOAP
     * notifications come into the SoapServlet, not into here.
     */
    public void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException {
	LibertyURLFederationTerminationNotification f = null;
	FederationTerminationNotificationProcessor p;

	try {
	    f = new LibertyURLFederationTerminationNotification();
	    f.parse(req.getQueryString());
	    p = new FederationTerminationNotificationProcessor();

	    RequestProcessorAction action;
	    action = p.process(f, getLibertySession(req));

	    ServletConfig cfg = getServletConfig();
	    ServletContext ctx = cfg.getServletContext();

	    if (action == RequestProcessorAction.ATTEMPT_USER_AUTH) {
		ctx.getNamedDispatcher("LoginServlet").forward(req, resp);
		return;
	    }
	} catch (Exception e) {
	    e.printStackTrace();
	}

	try {
	    String provID = f.getProviderID();
	    if (Config.debug) { System.err.println("ProviderID:" + provID); }
	    ProviderDescriptor prov = dir.find(provID);
	    URL returnURL = prov.getFederationTerminationReturnURL();
	    if (returnURL != null) {
		if (Config.debug) {
		    System.err.println("Return URL is: " + returnURL);
		}
		resp.sendRedirect(returnURL.toString());
	    }
	} catch (Exception e) {
	    throw new ServletException("Cannot redirect to return URL", e);
	}
    }
}
