/*
 * $Id: IDPLogoutCompletionServlet.java,v 1.3.2.2 2002/09/17 01:52:34 jsergent Exp $
 * IDPLogoutServlet.java
 *
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.experimentalstuff.com/Technologies/IPL/download.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2002 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): Ana Sanz Merino [University of California Berkeley, 2003]
 */

package com.sun.liberty.servlets;

import com.sun.liberty.Config;
import com.sun.liberty.LibertyUser;
import com.sun.liberty.LibertyUserSession;
import ucb.eecs.sahara.sso.authn.IPFilterUtils;
import java.io.IOException;
import java.util.List;
import javax.servlet.RequestDispatcher;
import javax.servlet.ServletContext;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpSession;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.Cookie;

public class IDPLogoutCompletionServlet extends HttpServlet {
    public void service(HttpServletRequest req, HttpServletResponse resp)
           throws ServletException {
        // Standard protected resource garbage copied from protected.jsp
        // (Should be in a servlet filter, of course...)
        HttpSession ses = req.getSession(true);
        LibertyUserSession lus;
        lus = (LibertyUserSession) ses.getAttribute("liberty.session");
        if (lus == null) {
            lus = new LibertyUserSession();
            ses.setAttribute("liberty.session", lus);
        }
        LibertyUser lu = lus.getUser();
        ServletConfig cfg = getServletConfig();
        ServletContext ctx = cfg.getServletContext();
        if (lu == null) {
            RequestDispatcher disp;
            disp = ctx.getNamedDispatcher("LoginServlet");
            try {
                disp.forward(req, resp);
            } catch (IOException e) {
                throw new ServletException("Cannot forward", e);
            }
            lus = (LibertyUserSession) ses.getAttribute("liberty.session");
            if (lus != null) {
                lu = lus.getUser();
            }
        }
        if (lus == null || lu == null) {
            return;
        }
	if (req.getParameter("webBugsDone") != null) {
	    lus.setPendingSLOWebBugs(false);
	}
	if (lus.getPendingSLOWebBugs()) {
	    try {
	        ctx.getRequestDispatcher("/web-bug.gif").forward(req, resp);
            } catch (IOException e) {
                throw new ServletException("Cannot forward", e);
            }
	}
        if (lus.getPendingSLO() != true) {
	    if (Config.debug) { System.err.println("SLO not pending!"); }
            try {
                ctx.getNamedDispatcher("IDPLogoutServlet").forward(req, resp);
            } catch (IOException e) {
                throw new ServletException("Cannot forward", e);
            }
            return;
        }
        if (lus.getPendingSLOImages().size() > 0) {
	    if (Config.debug) { System.err.println("Doing web bugs now"); }
            try {
		lus.setPendingSLOWebBugs(true);
                ctx.getRequestDispatcher("webbug-logout.jsp").forward(req,
                                                                      resp);
            } catch (IOException e) {
                throw new ServletException("Cannot forward", e);
            }
            return;
        }
        List redirs = lus.getPendingSLORedirects();
        if (redirs.size() > 0) {
	    if (Config.debug) { System.err.println("Doing redirs"); }
            String url = (String) redirs.get(0);
            redirs.remove(0);
            try {
	        if (Config.debug) { System.err.println("redir:" + url); }
                resp.sendRedirect(url);
            } catch (IOException e) {
                throw new ServletException("Cannot redirect", e);
            }            
            return;
        } else {
	    if (Config.debug) { System.err.println("No redirs"); }
            /* Actually log out the user here */
            lus.setUser((LibertyUser) null);
	    lus.setPendingSLO(false);

            /* Disallow user to access the external network */
            if (Config.ipManager != null) {
                if (Config.debug) {
                     System.err.println("removing remote IP from the filter");
                }
                String remoteIp = req.getRemoteAddr();
                if (!IPFilterUtils.removeIP(remoteIp)) {
                    throw new ServletException (
                        "Removing address from IP Filter failed");
                }
            }

            /* Remove the cookie with the IDP used for authentication */
            Cookie[] receivedCookies = req.getCookies();
            if (Config.debug) {System.err.println("read cookies");}
            if (receivedCookies != null) {
                int len = receivedCookies.length;
                for (int i=0; i < len; i++) {
                    if (receivedCookies[i].getName().equals("idp")) {
                        receivedCookies[i].setDomain(".cs.berkeley.edu");
                        receivedCookies[i].setMaxAge(0);
                        resp.addCookie(receivedCookies[i]);
                        if (Config.debug) {
                            System.err.println("\"idp\" cookie removed");
                        }
                    }
                }
            }
            else {
                if (Config.debug) {System.err.println("no cookies received");}
            }

            try {
                if (lus.getPendingRelayURL() != null) {
		    System.err.println("url:" + lus.getPendingRelayURL());
                    resp.sendRedirect(lus.getPendingRelayURL());
		    lus.setPendingRelayURL(null);
                } else {
                    ctx.getRequestDispatcher(
                        Config.logoutDonePage).forward(req, resp);
                }
            } catch (IOException e) {
                throw new ServletException("Cannot redirect", e);
            }
        
        }
    }
}
