/*
 * $Id: LogoutServlet.java,v 1.3.2.3 2002/09/17 01:52:34 jsergent Exp $
 * LogoutServlet.java
 *
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.experimentalstuff.com/Technologies/IPL/download.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2002 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 */

package com.sun.liberty.servlets;
import com.sun.liberty.Config;
import com.sun.liberty.ProviderDescriptorFactory;
import com.sun.liberty.ProviderDescriptor;
import com.sun.liberty.IDPDescriptor;
import com.sun.liberty.SPDescriptor;
import com.sun.liberty.LibertyUser;
import com.sun.liberty.LibertyUserSession;
import com.sun.liberty.LibertyURLLogoutNotification;
import com.sun.liberty.RequestProcessorAction;
import com.sun.liberty.LibertyLogoutNotification;
import com.sun.liberty.saml.SAMLNameIdentifier;
import java.io.IOException;
import java.net.URL;
import java.util.Iterator;
import java.util.Set;
import javax.servlet.RequestDispatcher;
import javax.servlet.ServletException;
import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

/**
 * Servlet to handle incoming front channel LogoutNotifications
 *
 * @author Jonathan Sergent
 * @version $Revision: 1.3.2.3 $
 */

public class LogoutServlet extends HttpServlet {
    /**
     * Utility function to locate user's LibertyUserSession object from the
     * HttpSession.  If none is found, a new one is created.
     */
    private static LibertyUserSession getLibertySession(HttpServletRequest req) {
        HttpSession ses = req.getSession(true);
        LibertyUserSession lus;
        lus = (LibertyUserSession) ses.getAttribute("liberty.session");
        if (lus == null) {
            lus = new LibertyUserSession();
            ses.setAttribute("liberty.session", lus);
        }
        return lus;
    }

    /**
     * Implements standard servlet doGet interface.  Handles incoming
     * LogoutNotifications that are URL-encoded.  SOAP
     * notifications come into the SoapServlet, not into here.
     */
    public void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException {
        LibertyURLLogoutNotification f = null;

        LibertyUserSession lus = getLibertySession(req);

        f = new LibertyURLLogoutNotification();
        try {
            f.parse(req.getQueryString());
        } catch (Exception e) {
            throw new ServletException("Cannot parse request", e);
        }

        ProviderDescriptorFactory fac = ProviderDescriptorFactory.getInstance();
        ProviderDescriptor sender = fac.find(f.getProviderID());
        boolean senderIsSP = sender instanceof SPDescriptor;
        String returnURL = sender.getSingleLogoutReturnURL().toString();

	if (f.getSignedBy() == null
	    || ! f.getSignedBy().equals(f.getProviderID()))
	{
	    if (Config.debug) {
		System.err.println("signature mismatch");
		System.err.println("signedBy:" + f.getSignedBy());
		System.err.println("providerID:" + f.getProviderID());
	    }
            try {
                resp.sendRedirect(returnURL);
            } catch (IOException e) {
                throw new ServletException("Cannot redirect", e);
            }
            return;
        }

        LibertyUser u = lus.getUser();
        if (u == null) {
	    if (Config.debug) { System.err.println("not logged in "); }
            try {
                resp.sendRedirect(returnURL);
            } catch (IOException e) {
                throw new ServletException("Cannot redirect", e);
            }
            return;
        }

        SAMLNameIdentifier nameid = f.getNameIdentifier();
        SAMLNameIdentifier loggedIn;
        try {
            if (senderIsSP) {
                loggedIn = u.getLocalNameFor((SPDescriptor) sender);
            } else {
                loggedIn = u.getLocalNameFor((IDPDescriptor) sender);
            }
        } catch (Exception e) {
            throw new ServletException("Cannot look up name", e);
        }
        if (Config.debug) { System.err.println("loggedIn:" + loggedIn); }
        if (loggedIn.toString().equals(nameid.toString())) {
	    if (Config.debug) { System.err.println("user is same"); }
            if (lus.getRelyingSPList() != null
			&& lus.getRelyingSPList().size() > 0) {
		if (Config.debug) { System.err.println("Relying SPs found -- do IDP logout stuff"); }
                /*
                 * FIXME: If sender is an IDP using GET profile, and
                 * one of the relying SPs is using the GET profile,
                 * this will break.  Should have way to remember this
                 * so that the IDPLogoutServlet doesn't do the wrong
                 * thing.
                 */
                lus.setPendingSLO(true);
                lus.setPendingRelayURL(returnURL);
                ServletConfig cfg = getServletConfig();
                ServletContext ctx = cfg.getServletContext();
                RequestDispatcher disp;

		/* Remove sender from list so we don't notify it */
		Set s = lus.getRelyingSPList();
		for (Iterator i = s.iterator(); i.hasNext(); ) {
		    SPDescriptor sp = (SPDescriptor) i.next();
		    if (sp.getProviderID().equals(sender.getProviderID())) {
		        s.remove(sp);
			break;
		    }
		}

                disp = ctx.getNamedDispatcher("IDPLogoutServlet");
                try {
                    disp.forward(req, resp);
                } catch (IOException e) {
                    throw new ServletException("Cannot forward", e);
                }
                return;
            } else {
		if (Config.debug) { System.err.println("No relying SPs -- just local logout"); }
                lus.setUser((LibertyUser) null);
            }
        }
        try {
            resp.sendRedirect(returnURL);
        } catch (IOException e) {
            throw new ServletException("Cannot redirect", e);
        }
    }
}
