/*
 * $Id: SLOServlet.java,v 1.1.2.2 2002/09/17 01:52:34 jsergent Exp $
 * LogoutServlet.java
 *
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.experimentalstuff.com/Technologies/IPL/download.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2002 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): Ana Sanz Merino [University of California Berkeley, 2003]
 */

package com.sun.liberty.servlets;
import com.sun.liberty.Config;
import com.sun.liberty.IDPDescriptor;
import com.sun.liberty.LibertyAuthenticationStatement;
import com.sun.liberty.LibertyAssertion;
import com.sun.liberty.LibertySoapLogoutNotification;
import com.sun.liberty.LibertyUser;
import com.sun.liberty.LibertyUserSession;
import com.sun.liberty.LibertyURLLogoutNotification;
import com.sun.liberty.ProviderDescriptorFactory;
import com.sun.liberty.SoapUtils;
import com.sun.liberty.saml.SAMLNameIdentifier;
import ucb.eecs.sahara.sso.authn.IPFilterUtils;
import java.io.IOException;
import java.util.List;
import javax.servlet.RequestDispatcher;
import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import javax.xml.soap.SOAPMessage;

/**
 *
 *
 * @author Jonathan Sergent
 * @version $Revision: 1.1.2.2 $
 */

public class SLOServlet extends HttpServlet {
    static ProviderDescriptorFactory dir;

    static {
	try {
	    dir = ProviderDescriptorFactory.getInstance();
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }

    /**
     * Utility function to locate user's LibertyUserSession object from the
     * HttpSession.  If none is found, a new one is created.
     */
    private static LibertyUserSession getLibertySession(HttpServletRequest req) {
        HttpSession ses = req.getSession(true);
        LibertyUserSession lus;
        lus = (LibertyUserSession) ses.getAttribute("liberty.session");
        if (lus == null) {
            lus = new LibertyUserSession();
            ses.setAttribute("liberty.session", lus);
        }
	return lus;
    }

    public void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
	LibertyUserSession lus = getLibertySession(req);
	String nonce = req.getParameter("nonce");

	if (nonce == null || !(nonce.equals(lus.getFCNonce()))) {
	    resp.sendRedirect(Config.logoutPageForRedirect);
	    return;
	}

        HttpSession ses = req.getSession(true);
	LibertyUser lu = lus.getUser();
	if ((lu == null) &&
            (ses.getAttribute("radius.authenticated") == null)) {
	    throw new ServletException("Current user is null");
	}

	lus.setPendingRelayURL(null);
        if (ses.getAttribute("radius.authenticated") != null) {
            ses.removeAttribute("radius.authenticated");
        }

        ServletConfig cfg = getServletConfig();
	ServletContext ctx = cfg.getServletContext();
	RequestDispatcher d;
        if (lu != null) {
	    try {
	        if (sendLogoutNotification(lus, lu, resp)) {
		    if (Config.debug) {
                        System.err.println("Will complete later");
                    }
		    lus.newFCNonce();
		    return;
	        }
	    } catch (Exception e) {
	        throw new ServletException("Cannot send logout to IDP", e);
	    }
	    d = ctx.getNamedDispatcher("IDPLogoutServlet");
	    if (Config.debug) {
                System.err.println("On to IDPLogoutServlet");
            }
	    d.forward(req, resp);
	    lus.newFCNonce();
        }
        else {
            /* Disallow user to access the external network */
            if (Config.ipManager != null) {
                if (Config.debug) {
                     System.err.println("removing remote IP from the filter");
                }
                String remoteIp = req.getRemoteAddr();
                if (!IPFilterUtils.removeIP(remoteIp)) {
                    throw new ServletException ("Removing address from IP Filter failed");
                }
            }
            d = ctx.getRequestDispatcher(Config.logoutDonePage);
	    if (Config.debug) {
                System.err.println("On to Logout-done Page");
            }
	    d.forward(req, resp);
        }
    }

    private boolean sendLogoutNotification(LibertyUserSession lus, LibertyUser lu, HttpServletResponse resp) throws Exception {
	if (Config.debug) { System.err.println("Notifying IDP"); }
	LibertyAssertion assertion = lus.getAuthnAssertion();
	if (assertion == null) { return false; }
	/* We need to send logout notification to an IDP first */
	String remoteID = assertion.getIssuer();
	List statements = assertion.getStatements();
	if (statements == null) { return false; }
	LibertyAuthenticationStatement stmt;
	stmt = (LibertyAuthenticationStatement) statements.get(0);
	if (stmt == null) { return false; }
	String idpSessionIndex = stmt.getSessionIndex();
	IDPDescriptor idp = dir.findIDP(remoteID);
	if (idp == null) { return false; }
	SAMLNameIdentifier nameid = lu.getNameProvidedBy(idp);
	if (nameid == null) { return false; }

	LibertySoapLogoutNotification n;
	n = new LibertySoapLogoutNotification();
	n.setProviderID(Config.localProviderID);
	n.setSignedBy(Config.localProviderID);
	n.setSessionIndex(idpSessionIndex);
	n.setNameIdentifier(nameid);
	lus.setPendingSLO(true);
	SOAPMessage r = SoapUtils.send(n, idp.getSoapEndpoint().toString());

	if (r != null && !r.getSOAPPart().getEnvelope().getBody().hasFault()) {
	    /*
	     * SOAP was successful -- return false since we should go to
	     * the IDPLogoutServlet immediately
	     */
	    if (Config.debug) { System.err.println("SOAP succeeded"); }
	    return false;
	}

	if (Config.debug) { System.err.println("SLOServlet: Trying FC."); }
	LibertyURLLogoutNotification f = new LibertyURLLogoutNotification();
	f.setProviderID(Config.localProviderID);
	f.setSignedBy(Config.localProviderID);
	f.setSessionIndex(idpSessionIndex);
	f.setNameIdentifier(nameid);
	lus.setPendingSLO(true);
	resp.sendRedirect(idp.getSingleLogoutURL() + "?" + f.getQuery());
	return true;
    }
}
