package ucb.eecs.sahara.sso;

import com.sun.liberty.Config;
import java.io.*;
import java.util.Timer;
import java.util.TimerTask;


/**
 * Utilities for debugging.
 *
 * @author Ana Sanz Merino [University of California Berkeley, 2003]
 * @version $Revision: 1.1.1.1 $
 */


public class DebugUtils {

    public static final int HIGH_LEVEL_FUNC = 1;
    public static final int LOW_LEVEL_FUNC = 3;

    static final String outFileName = Config.timesFile;
    static PrintWriter out;

    static {
	if (Config.timeDebug) {
            try {
                out = new PrintWriter(new BufferedWriter(new FileWriter(outFileName)));
                /* Flush the stream after certain time interval */
                //Create timer as daemon, so that it dies when the
                //invoking thread dies
                Timer timer = new Timer(true);
                DebugFlushTimeout task = new DebugFlushTimeout();
                timer.schedule(task, Config.debugFlushTimeout * 1000);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }


    protected void finalize() throws Throwable {
        try {
            out.close();
        } finally {
            super.finalize();
        }
    }


    public static void debugTime(String component, String msg, int level) {
        if (Config.timeDebug && level <= Config.debugLevel) {
            try {
                out.println("MESSAGE :" + component + ": " + msg + " at:");
                out.println("TIME " + new java.util.Date().getTime());
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    public static void flush() {
        try {
            out.flush();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}


class DebugFlushTimeout extends TimerTask {
    public void run () {
        DebugUtils.flush();

        /* Flush the stream after certain time interval */
        //Create timer as daemon, so that it dies when the
        //invoking thread dies
        Timer timer = new Timer(true);
        DebugFlushTimeout task = new DebugFlushTimeout();
        timer.schedule(task, Config.debugFlushTimeout * 1000);
    }
}


