package ucb.eecs.sahara.sso.anp;

import ucb.eecs.sahara.sso.anp.ANPElement;
import ucb.eecs.sahara.sso.anp.saml.SAMLPQuery;
import java.util.List;
import java.util.LinkedList;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * Data bindings for anp:AuthnQuery element.  Get and set
 * methods correspond to child elements and attributes in the XML schema
 * for this element.
 *
 * @author Ana Sanz Merino
 * @version $Revision: 1.1.1.1 $
 */

public class ANPAuthnQuery extends SAMLPQuery implements ANPElement{
    String authnMethodIDReference = null;
    String chargingOptionIDReference = null;
    List userInfo = null;


    public void parse(Element e) {
	NodeList list = e.getChildNodes();
	for (int j = 0; j < list.getLength(); j++) {
	    if (list.item(j) instanceof org.w3c.dom.Text) { continue; }
	    Element child = (Element) list.item(j);
	    if (isElement(child, ANP_NS_URI, "AuthnMethodIDReference")) {
                setAuthnMethodIDReference(child.getFirstChild().getNodeValue());
	    } else if (isElement(child, ANP_NS_URI, "ChargingOptionIDReference")) {
                setChargingOptionIDReference(child.getFirstChild().getNodeValue());
	    } else if (isElement(child, ANP_NS_URI, "UserInfo")) {
		ANPUserInfo c = new ANPUserInfo();
		c.parse(child);
		List set = getUserInfo();
		if (set == null) {
		    set = new LinkedList();
		}
		set.add(c);
		setUserInfo(set);
	    }
	}
    }

    public Element getElement(Document doc) {
	Element s;
	s = doc.createElementNS(ANP_NS_URI, ANP_NS_ID + ":AuthnQuery");
	addChildrenTo(s);
	return s;
    }

    protected void addChildrenTo(Element s) {
        appendNewChildNSWithText(s, ANP_NS_URI, 
            ANP_NS_ID + ":AuthnMethodIDReference", authnMethodIDReference);
        appendNewChildNSWithText(s, ANP_NS_URI, 
            ANP_NS_ID + ":ChargingOptionIDReference", chargingOptionIDReference);
	iterateAddChildren(s, userInfo);
    }

    public String getAuthnMethodIDReference() {
	return authnMethodIDReference;
    }

    public void setAuthnMethodIDReference(String value) {
	authnMethodIDReference = value;
    }

    public String getChargingOptionIDReference() {
	return chargingOptionIDReference;
    }

    public void setChargingOptionIDReference(String value) {
	chargingOptionIDReference = value;
    }

    public List getUserInfo() {
	return userInfo;
    }

    public void setUserInfo(List value) {
	userInfo = value;
    }
}
