package ucb.eecs.sahara.sso.anp;

import com.sun.liberty.Config;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import java.util.List;
import java.util.Iterator;

import ucb.eecs.sahara.sso.authn.SimpleLocalAuthenticator;
import ucb.eecs.sahara.sso.authn.AuthenticatorResult;


/**
 * @author Ana Sanz Merino [University of California Berkeley, 2003]
 * @version $Revision: 1.1.1.1 $
 */

public class SimpleLocalANPAuthnQueryProcessor implements ANPAuthnQueryProcessor
{

    private static final String loginParam = "loginInfo/userName";
    private static final String passwordParam = "loginInfo/password";

    public ANPRequestProcessorAction process(HttpServletRequest httpReq,
					     HttpServletResponse httpResp,
					     ANPAuthnQuery authnQuery,
                                             ANPAuthnQueryProcResult procResult)
    {
    try {
        String userName = null;
        String userPassword = null;

        /* Extract user information from the ANPAuthnQuery */
        Iterator i = null;
        List userInfo = authnQuery.getUserInfo();
        i = userInfo.iterator();
        while (i.hasNext()) {
            ANPUserInfo ui = (ANPUserInfo) i.next();
            if (ui.getAttributeName().equals(loginParam)) {
                userName = (String) ui.getAttributeValues().iterator().next();
            }
            if (ui.getAttributeName().equals(passwordParam)) {
                userPassword = 
		    (String) ui.getAttributeValues().iterator().next();
            }
        }
        if (Config.debug) {
            System.err.println("LOCAL requested for " + userName);
        }

        /* Check all required user info was present in the ANPAuthnQuery */
        if ((userName == null ) || (userPassword == null)) {
            if (Config.debug) {
                System.err.println("Missing required user information for LOCAL authentication");
            }
            procResult.failureCode = "samlp:Sender";
            return ANPRequestProcessorAction.RETURN_RESPONSE;
        }

        /* Authenticate the user */
        AuthenticatorResult authnResult = 
            SimpleLocalAuthenticator.authenticate(userName, userPassword);

        if (authnResult == AuthenticatorResult.AUTHN_SUCCESS) {
            procResult.authnQueryResult = new ANPAuthnQueryResult();
            procResult.authnQueryResult.authnResult = "Successful";
        } else if (authnResult == AuthenticatorResult.AUTHN_FAILED) {
            procResult.authnQueryResult = new ANPAuthnQueryResult();
            procResult.authnQueryResult.authnResult = "Failed";
        } else {
            if (Config.debug) {
                System.err.println("Error at LOCAL authentication");
            }
            procResult.failureCode = "samlp:Receiver";
	}
        return ANPRequestProcessorAction.RETURN_RESPONSE;
    } catch (Exception e) {
        if (Config.debug) {
            System.err.println(e);
        }
        return ANPRequestProcessorAction.FAILED;
    }
    }
}
