package ucb.eecs.sahara.sso.authn;

import ucb.eecs.sahara.sso.DebugUtils;

import com.sun.liberty.Config;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import java.util.Date;
import java.util.Iterator;
import java.util.List;

import com.sun.liberty.IDPDescriptor;
import com.sun.liberty.LibertyProfile;
import com.sun.liberty.LibertyUser;
import com.sun.liberty.LibertyURLAuthnRequest;
import com.sun.liberty.LibertyUserSession;
import com.sun.liberty.ProviderDescriptor;
import com.sun.liberty.ProviderDescriptorFactory;
import com.sun.liberty.ArtifactUtils;
import com.sun.liberty.LibertyAssertion;
import com.sun.liberty.LibertyAuthnRequest;
import com.sun.liberty.LibertyAuthenticationStatement;
import com.sun.liberty.XMLUtils;

import com.sun.liberty.saml.SAMLAudienceRestrictionCondition;
import com.sun.liberty.saml.SAMLAuthenticationStatement;
import com.sun.liberty.saml.SAMLCondition;
import com.sun.liberty.saml.SAMLConditions;
import com.sun.liberty.saml.SAMLNameIdentifier;
import com.sun.liberty.saml.SAMLSubject;
import com.sun.liberty.saml.SAMLPResponse;


/**
 * @author Ana Sanz Merino
 * @version $Revision: 1.1.1.1 $
 */

public class LibertyAuthenticator {

    static ProviderDescriptorFactory dir;

    static {
        try {
            dir = ProviderDescriptorFactory.getInstance();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


    private static LibertyUserSession getLibertySession(HttpServletRequest httpReq) {
        HttpSession ses = httpReq.getSession(true);
        LibertyUserSession lus;
        lus = (LibertyUserSession) ses.getAttribute("liberty.session");
        if (lus == null) {
            lus = new LibertyUserSession();
            ses.setAttribute("liberty.session", lus);
        }
        return lus;
    }


    public static void sessionCleanUp(HttpServletRequest httpReq) {
        if (Config.debug) {
            System.err.println("Entering LibertyAuthenticator sessionCleanUp");
        }
        HttpSession ses = httpReq.getSession(true);
        if (ses.getAttribute("liberty.session") != null) {
            ses.removeAttribute("liberty.session");
        }
    }


    public static AuthenticatorResult authenticate(HttpServletRequest httpReq,
                                                   final String idpName,
						   String redirectURL[])
                  throws Exception
    {
        try {
            ProviderDescriptorFactory d;
            d = ProviderDescriptorFactory.getInstance();
            IDPDescriptor i = (IDPDescriptor) d.find(idpName);
            if (i == null) {
	        return AuthenticatorResult.AUTHN_FAILED;
            }
            ProviderDescriptor l = d.findLocal();

            LibertyURLAuthnRequest ar = new LibertyURLAuthnRequest();
            ar.setProviderID(l.getProviderID());
            ar.setIsPassive(false);
            ar.setForceAuthn(false);
            ar.setFederate(false);
            if (Config.useFormPost) {
                ar.setProtocolProfile(LibertyProfile.BRWS_POST);
            }

            /*
             * XXX: Do the LECP dance
             * Check req for header; if present, send back a magic LECP
             * response here.
             */
            String url = i.getSingleSignOnURL() + "?" + ar.getQuery();
            if (Config.debug) { System.err.println("url:" + url); }

            /*
             * IDP will send user back to our assertion consumer URL with
             * an AuthnResponse.
             * This session parameter will tell the assertion consumer
             * servlet where to send the user to after checking the
             * AuthnResponse.
             */
            LibertyUserSession lus = getLibertySession(httpReq);
            lus.setPendingAuthnRequest(ar);
            lus.setPendingIDP(idpName);

            /* If a successful authentication page is defined, substitute
               the relay page by it */
            if (Config.successfulLibertyAuthnPage != null) {
                lus.setPendingRelayURL(Config.localProviderApp + 
                                       Config.successfulLibertyAuthnPage);
            }
            else {
                lus.setPendingRelayURL(httpReq.getRequestURL().toString());
            }

            redirectURL[0] = url;
            redirectURL[1] = i.getSingleSignOnURL().toString();
            return AuthenticatorResult.AUTHN_REDIRECT;
	}
        catch (Exception e) {
            e.printStackTrace();
            throw new Exception("LibertyAuthenticator - Liberty IDP Authentication- failed", e);
        }
    }


    public static AuthenticatorResult authenticateArtifact(HttpServletRequest httpReq,
                                                           String artifact,
                                                           LibertyAssertionControl aCtl)
                  throws Exception
    {
         LibertyAssertion assertion = extractAssertion(artifact);
         return authenticateAssertion(httpReq, assertion, aCtl);
    }


    /**
     * Validates assertion. Performs federation if the pending AuthnRequest
     *  has Federate set to true.
     * If the incoming assertion is valid,
     * the user's session will be updated with the LibertyUser object located by the
     * NameIdentifier in the assertion.  If the pending AuthnRequest had Federate
     * set to true, a federation entry will be made for the user based on the
     * NameIdentifiers in the assertion.
     *
     * @param   httpReq         Current servlet request
     * @param   a               The received assertion
     */
 
    public static AuthenticatorResult authenticateAssertion(HttpServletRequest httpReq,
                                                            LibertyAssertion a,
                                                            LibertyAssertionControl aCtl)
                  throws Exception
    {
        DebugUtils.debugTime("LibertyAuthenticator", "Entering authenticateAssertion",
                             DebugUtils.LOW_LEVEL_FUNC);

        /* Initialize the assertion control information */
        if (aCtl == null) {
            if (Config.debug) {
                System.err.println("Received a null LibertyAssertionControl");
            }
            return AuthenticatorResult.AUTHN_FAILED;
        }
        aCtl.federate = false;
        aCtl.redirectedServer = null;

        /* Retrieve the session information */
        LibertyUserSession ses = getLibertySession(httpReq);
        LibertyAuthnRequest req = ses.getPendingAuthnRequest();
        String idpid = ses.getPendingIDP();

        if (Config.debug) {
            System.err.println("pending idpid:" + idpid);
            try {
                System.err.println("pending req:" + XMLUtils.serialize(req));
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        ses.setPendingAuthnRequest(null);


        /* Check whether the assertion is valid */
        boolean valid = true;

        if (req != null && a != null && idpid != null) {
            long now = new Date().getTime();
            long issueInstantTime = a.getIssueInstant().getTime();
            SAMLAuthenticationStatement s;
            s = (SAMLAuthenticationStatement) a.getStatements().get(0);
             long authnInstantTime = s.getAuthenticationInstant().getTime();
            if (!idpid.equals(a.getIssuer())) {
                if (Config.debug) { System.err.println("Unexpected issuer"); }
                valid = false;
            } else if (!idpid.equals(a.getSignedBy())) {
                if (Config.debug) {
                    System.err.println("Not signed properly");
                    System.err.println("a signedBy:" + a.getSignedBy());
                }
                valid = false;
            } else if (!req.getRequestID().equals(a.getInResponseTo())) {
                if (Config.debug) { System.err.println("Bad InResponseTo"); }
                valid = false;
            } else if (checkConditions(a.getConditions()) == false) {
                if (Config.debug) { System.err.println("Conditions failed"); }
                valid = false;
            } else if (Math.abs(now - issueInstantTime)
                       > Config.issueInstantTolerance) {
                if (Config.debug) { System.err.println("Bad issue instant"); }
                valid = false;
            } else if (Math.abs(now - authnInstantTime)
                       > Config.issueInstantTolerance) {
                if (Config.debug) { System.err.println("Bad authn instant"); }
                valid = false;
            }
            /* XXX: Should optionally check SubjectLocality. */
        } else {
            if (Config.debug) {
                System.err.println("Missing data");
                System.err.println("req:" + req);
                System.err.println("a:" + a);
                System.err.println("idpid:" + idpid);
            }
            valid = false;
        }
        DebugUtils.debugTime("LibertyAuthenticator", "Validation finished",
                             DebugUtils.LOW_LEVEL_FUNC);

        /* If the assertion is valid, process it */
        if (valid) {
            if (Config.debug) { System.err.println("Handling assertion..."); }

            String issuer = a.getIssuer();
            IDPDescriptor idp;
            idp = (IDPDescriptor) dir.find(issuer);

            aCtl.redirectedServer = (String) idp.getSingleSignOnURL().toString();

            List statements = a.getStatements();
            LibertyAuthenticationStatement s;
            s = (LibertyAuthenticationStatement) statements.iterator().next();
            SAMLSubject subj = s.getSubject();
            /* The one we provided: */
            SAMLNameIdentifier name = subj.getNameIdentifier();

            if (req.getFederate() == true) {
                if (Config.debug) { System.err.println("Trying to federate."); }

                aCtl.federate = true;

                /* Federate... */
                LibertyUser user = ses.getUser();
                if (user == null) {
                    /*
                     * XXX: Should optionally authn the user and then federate
                     * or create a new user here if the admin wants it to work
                     * that way.
                     *
                     * Not supported for now.
                     */
                    if (Config.debug) {
                        System.err.println("Cannot federate unknown user");
                    }
	            return AuthenticatorResult.AUTHN_FAILED;
                } else {
                    try {
                        if (user.getNameProvidedBy(idp) != null
                                || user.getLocalNameFor(idp) != null) {
                            throw new Exception("Duplicate federation");
                        }
                    } catch (Exception e) {
                        /* Okay for these to fail */
                    }
                    if (Config.debug) {
                        System.err.println("Federating current user...");
                    }
                    try {
                        user.setFederationStatus(idp, true);
                        user.setNameProvidedBy(idp, name);
                        user.setLocalNameFor(idp, name);
                    } catch (Exception e) {
                        e.printStackTrace();
                        if (Config.debug) {
                            System.err.println("Federation failed");
                        }
	                return AuthenticatorResult.AUTHN_FAILED;
                    }
                    ses.setAuthnAssertion(a);
                    return AuthenticatorResult.AUTHN_SUCCESS;
                }
            } else {
                // FIXME: find is actually backwards... should take sp provided
                // name when finding a federated identity, and idp provided name
                // when finding a federated service.  Claims to take IDP
                // provided name but we aren't giving that to it.  Same for
                // now.
                try {
                    if (Config.debug) {
                        System.err.println("Trying to find user...");
                    }
                    ses.setUser(idp, name);
                    ses.setAuthnAssertion(a);
                    if (Config.debug) {
                        System.err.println("Now have user: " + ses.getUser());
                    }
                    return AuthenticatorResult.AUTHN_SUCCESS;
                } catch (Exception e) {
                    /*
                     * The IDP thinks we're federated, but we don't think we
                     * are...
                     */
                    if (Config.debug) {
                         System.err.println("Can't find federated user...");
                    }
                    e.printStackTrace();
	            return AuthenticatorResult.AUTHN_FAILED;
                }
            }
        } else {
            if (Config.debug) {
                System.err.println("valid == false...");
            }
	    return AuthenticatorResult.AUTHN_FAILED;
        }
    }


    private static LibertyAssertion extractAssertion(String art)
                                    throws Exception
    {
        if (art == null) {
            throw new Exception("LibertyAuthenticator - Artifact is not present");
        }

        SAMLPResponse response;
        try {
            response = ArtifactUtils.dereferenceArtifact(art);
        } catch (Exception e) {
            throw new Exception("LibertyAuthenticator - Could not get assertion", e);
        }

        List assertions = response.getAssertions();
        if (assertions == null) {
            if (Config.debug) {
                System.err.println("LibertyAuthenticator: assertions == null...");
            }
            return null;
        }

        LibertyAssertion a;
        a = (LibertyAssertion) assertions.iterator().next();
        return a;
    }

    private static boolean checkConditions(SAMLConditions conditions) {
        if (conditions == null) {
            return true;
        }
        /* Checks date and time attrs on <Conditions> */
        long now = new Date().getTime();
        Date notBefore = conditions.getNotBefore();
        if (notBefore != null) {
            if (notBefore.getTime() > now) {
                if (Config.debug) { System.err.println("NotBefore condition failed"); }
                return false;
            }
        }
        Date notOnOrAfter = conditions.getNotOnOrAfter();
        if (notOnOrAfter != null) {
            if (now >= notOnOrAfter.getTime()) {
                if (Config.debug) { System.err.println("NotOnOrAfter condition failed"); }
                return false;
            }
        }
        /*
         * Checks Condition elements inside <Conditions>
         * We only implement AudienceRestrictionCondition.
         * If there are others we don't grok, we must reject the assertion.
         */
        List condList = conditions.getConditions();
        if (condList != null) {
            Iterator i = condList.iterator();
            while (i.hasNext()) {
                SAMLCondition condition = (SAMLCondition) i.next();
                if (condition instanceof SAMLAudienceRestrictionCondition) {
                    SAMLAudienceRestrictionCondition arc;
                    arc = (SAMLAudienceRestrictionCondition) condition;
                    List audiences = arc.getAudiences();
                    if (audiences != null) {
                        String ourID = Config.localProviderID;
                        boolean found = false;
                        Iterator j = audiences.iterator();
                        while (j.hasNext()) {
                            String audience = (String) j.next();
                            if (audience.equals(ourID)) {
                                found = true;
                                break;
                            }
                        }
                        if (!found) {
                            if (Config.debug) { System.err.println("SAMLAudienceRestrictionCondition failed"); }
                            return false;
                        }
                    }
                } else {
                    /* Unrecognized condition */
                    if (Config.debug) { System.err.println("Unrecognized condition"); }
                    return false;
                }
            }
        }
        return true;
    }


}
