package ucb.eecs.sahara.sso.sp;

/**
 * Interface which allows the service provider obtain information
 * about the communication mechanism used by the client for authentication and 
 * expectations from the client after authentication.
 * 
 * @author Ana Sanz Merino [University of California Berkeley, 2003]
 * @version $Revision: 1.1.1.1 $
 */

public class AuthnControlSession {
    private boolean externalAccess = false;
    private String authnMethod = null;
    private String chargingOption = null;
    private String pendingRelayURL = null;

    /**
     * Returns the URL the client should be redirected to after
     * processing of the authentication request is complete.
     * Note: In the case of redirection, completetion is not reached until the 
     * incoming assertion is processed
     *
     * @return the relay URL, or null if none
     */
    public String getPendingRelayURL() {
	return pendingRelayURL;
    }

    /**
     * Sets or clears the URL that the server should redirect
     * the client to upon completely processing an authentication 
     * request.
     * Note: In the case of redirection, completetion is not reached until the 
     * incoming assertion is processed
     * @param value the new URL, or null
     */
    public void setPendingRelayURL(String value) {
	pendingRelayURL = value;
    }

    /**
     * Gets whether the user has requested access to the external network 
     * through the Service Provider or not.
     *
     * @return  True if the user has requested access to the exterior.
     */
    public boolean getExternalAccess() {
        return externalAccess;
    }

    /**
     * Stores whether the user has requested access to the external network
     * through the Service Provider or not.
     * @param externalAccess True means that the user has requested access to
     *                       the exterior.
     */
    public void setExternalAccess(boolean externalAccess) {
        this.externalAccess = externalAccess;
    }

    /**
     * Gets the authentication method selected by the client.
     *
     * @return  The authentication method selected, or null if it has not been set.
     */
    public String getAuthnMethod() {
        return authnMethod;
    }

    /**
     * Stores the authentication method selected by the client. 
     * @param authnMethod The authentication method selected.
     */
    public void setAuthnMethod(String authnMethod) {
        this.authnMethod = authnMethod;
    }

    /**
     * Gets the charging option selected by the client.
     *
     * @return  The charging option selected, or null if it has not been set.
     */
    public String getChargingOption() {
        return chargingOption;
    }

    /**
     * Stores the charging option selected by the client. 
     * @param chargingOption The charging option selected.
     */
    public void setChargingOption(String chargingOption) {
        this.chargingOption = chargingOption;
    }

}
